@@ -13,11 +13,22 @@ from sqlspec.config import (
     SyncConfigT,
     SyncDatabaseConfig,
 )
+from sqlspec.core.cache import (
+    CacheConfig,
+    get_cache_config,
+    get_cache_statistics,
+    log_cache_stats,
+    reset_cache_stats,
+    update_cache_config,
+)
 from sqlspec.utils.logging import get_logger
 
 if TYPE_CHECKING:
     from contextlib import AbstractAsyncContextManager, AbstractContextManager
+    from pathlib import Path
 
+    from sqlspec.core.statement import SQL
+    from sqlspec.loader import SQLFileLoader
     from sqlspec.typing import ConnectionT, PoolT
 
 
@@ -27,47 +38,76 @@ logger = get_logger()
 
 
 class SQLSpec:
-    """Type-safe configuration manager and registry for database connections and pools."""
+    """Configuration manager and registry for database connections and pools."""
 
-    __slots__ = ("_configs",)
+    __slots__ = ("_configs", "_instance_cache_config", "_sql_loader")
 
-    def __init__(self) -> None:
+    def __init__(self, *, loader: "Optional[SQLFileLoader]" = None) -> None:
         self._configs: dict[Any, DatabaseConfigProtocol[Any, Any, Any]] = {}
-        atexit.register(self._cleanup_pools)
+        atexit.register(self._cleanup_sync_pools)
+        self._instance_cache_config: Optional[CacheConfig] = None
+        self._sql_loader: Optional[SQLFileLoader] = loader
 
     @staticmethod
     def _get_config_name(obj: Any) -> str:
         """Get display name for configuration object."""
-        # Try to get __name__ attribute if it exists, otherwise use str()
         return getattr(obj, "__name__", str(obj))
 
-    def _cleanup_pools(self) -> None:
-        """Clean up all registered connection pools."""
+    def _cleanup_sync_pools(self) -> None:
+        """Clean up only synchronous connection pools at exit."""
         cleaned_count = 0
 
+        for config_type, config in self._configs.items():
+            if config.supports_connection_pooling and not config.is_async:
+                try:
+                    config.close_pool()
+                    cleaned_count += 1
+                except Exception as e:
+                    logger.debug("Failed to clean up sync pool for config %s: %s", config_type.__name__, e)
+
+        if cleaned_count > 0:
+            logger.debug("Sync pool cleanup completed. Cleaned %d pools.", cleaned_count)
+
+    async def close_all_pools(self) -> None:
+        """Explicitly close all connection pools (async and sync).
+
+        This method should be called before application shutdown for proper cleanup.
+        """
+        cleanup_tasks = []
+        sync_configs = []
+
         for config_type, config in self._configs.items():
             if config.supports_connection_pooling:
                 try:
                     if config.is_async:
                         close_pool_awaitable = config.close_pool()
                         if close_pool_awaitable is not None:
-                            try:
-                                loop = asyncio.get_running_loop()
-                                if loop.is_running():
-                                    _task = asyncio.ensure_future(close_pool_awaitable, loop=loop)  # noqa: RUF006
-
-                                else:
-                                    asyncio.run(cast("Coroutine[Any, Any, None]", close_pool_awaitable))
-                            except RuntimeError:  # No running event loop
-                                asyncio.run(cast("Coroutine[Any, Any, None]", close_pool_awaitable))
+                            cleanup_tasks.append(cast("Coroutine[Any, Any, None]", close_pool_awaitable))
                     else:
-                        config.close_pool()
-                    cleaned_count += 1
+                        sync_configs.append((config_type, config))
                 except Exception as e:
-                    logger.warning("Failed to clean up pool for config %s: %s", config_type.__name__, e)
+                    logger.debug("Failed to prepare cleanup for config %s: %s", config_type.__name__, e)
 
-        self._configs.clear()
-        logger.info("Pool cleanup completed. Cleaned %d pools.", cleaned_count)
+        if cleanup_tasks:
+            try:
+                await asyncio.gather(*cleanup_tasks, return_exceptions=True)
+                logger.debug("Async pool cleanup completed. Cleaned %d pools.", len(cleanup_tasks))
+            except Exception as e:
+                logger.debug("Failed to complete async pool cleanup: %s", e)
+
+        for _config_type, config in sync_configs:
+            config.close_pool()
+
+        if sync_configs:
+            logger.debug("Sync pool cleanup completed. Cleaned %d pools.", len(sync_configs))
+
+    async def __aenter__(self) -> "SQLSpec":
+        """Async context manager entry."""
+        return self
+
+    async def __aexit__(self, _exc_type: Any, _exc_val: Any, _exc_tb: Any) -> None:
+        """Async context manager exit with automatic cleanup."""
+        await self.close_all_pools()
 
     @overload
     def add_config(self, config: "SyncConfigT") -> "type[SyncConfigT]":  # pyright: ignore[reportInvalidTypeVarUse]
@@ -84,11 +124,11 @@ class SQLSpec:
             config: The configuration instance to add.
 
         Returns:
-            The type of the added configuration, annotated with its ID for potential use in type systems.
+            The type of the added configuration for use as a registry key.
         """
         config_type = type(config)
         if config_type in self._configs:
-            logger.warning("Configuration for %s already exists. Overwriting.", config_type.__name__)
+            logger.debug("Configuration for %s already exists. Overwriting.", config_type.__name__)
         self._configs[config_type] = config
         return config_type
 
@@ -233,14 +273,13 @@ class SQLSpec:
             async def _create_driver_async() -> "DriverT":
                 resolved_connection = await connection_obj  # pyright: ignore
                 return cast(  # pyright: ignore
-                    "DriverT",
-                    config.driver_type(connection=resolved_connection, default_row_type=config.default_row_type),
+                    "DriverT", config.driver_type(connection=resolved_connection)
                 )
 
             return _create_driver_async()
 
         return cast(  # pyright: ignore
-            "DriverT", config.driver_type(connection=connection_obj, default_row_type=config.default_row_type)
+            "DriverT", config.driver_type(connection=connection_obj)
         )
 
     @overload
@@ -291,7 +330,6 @@ class SQLSpec:
             *args: Positional arguments to pass to the config's provide_connection.
             **kwargs: Keyword arguments to pass to the config's provide_connection.
 
-
         Returns:
             A sync or async context manager yielding a connection.
         """
@@ -473,3 +511,178 @@ class SQLSpec:
 
         logger.debug("Config %s does not support connection pooling - nothing to close", config_name)
         return None
+
+    @staticmethod
+    def get_cache_config() -> CacheConfig:
+        """Get the current global cache configuration.
+
+        Returns:
+            The current cache configuration.
+        """
+        return get_cache_config()
+
+    @staticmethod
+    def update_cache_config(config: CacheConfig) -> None:
+        """Update the global cache configuration.
+
+        Args:
+            config: The new cache configuration to apply.
+        """
+        update_cache_config(config)
+
+    @staticmethod
+    def get_cache_stats() -> "dict[str, Any]":
+        """Get current cache statistics.
+
+        Returns:
+            Cache statistics object with detailed metrics.
+        """
+        return get_cache_statistics()
+
+    @staticmethod
+    def reset_cache_stats() -> None:
+        """Reset all cache statistics to zero."""
+        reset_cache_stats()
+
+    @staticmethod
+    def log_cache_stats() -> None:
+        """Log current cache statistics using the configured logger."""
+        log_cache_stats()
+
+    @staticmethod
+    def configure_cache(
+        *,
+        sql_cache_size: Optional[int] = None,
+        fragment_cache_size: Optional[int] = None,
+        optimized_cache_size: Optional[int] = None,
+        sql_cache_enabled: Optional[bool] = None,
+        fragment_cache_enabled: Optional[bool] = None,
+        optimized_cache_enabled: Optional[bool] = None,
+    ) -> None:
+        """Update cache configuration with partial values.
+
+        Args:
+            sql_cache_size: Size of the SQL statement cache.
+            fragment_cache_size: Size of the AST fragment cache.
+            optimized_cache_size: Size of the optimized expression cache.
+            sql_cache_enabled: Enable/disable SQL cache.
+            fragment_cache_enabled: Enable/disable fragment cache.
+            optimized_cache_enabled: Enable/disable optimized cache.
+        """
+        current_config = get_cache_config()
+        update_cache_config(
+            CacheConfig(
+                sql_cache_size=sql_cache_size if sql_cache_size is not None else current_config.sql_cache_size,
+                fragment_cache_size=fragment_cache_size
+                if fragment_cache_size is not None
+                else current_config.fragment_cache_size,
+                optimized_cache_size=optimized_cache_size
+                if optimized_cache_size is not None
+                else current_config.optimized_cache_size,
+                sql_cache_enabled=sql_cache_enabled
+                if sql_cache_enabled is not None
+                else current_config.sql_cache_enabled,
+                fragment_cache_enabled=fragment_cache_enabled
+                if fragment_cache_enabled is not None
+                else current_config.fragment_cache_enabled,
+                optimized_cache_enabled=optimized_cache_enabled
+                if optimized_cache_enabled is not None
+                else current_config.optimized_cache_enabled,
+            )
+        )
+
+    def load_sql_files(self, *paths: "Union[str, Path]") -> None:
+        """Load SQL files from paths or directories.
+
+        Args:
+            *paths: One or more file paths or directory paths to load.
+        """
+        if self._sql_loader is None:
+            from sqlspec.loader import SQLFileLoader
+
+            self._sql_loader = SQLFileLoader()
+
+        self._sql_loader.load_sql(*paths)
+        logger.debug("Loaded SQL files: %s", paths)
+
+    def add_named_sql(self, name: str, sql: str, dialect: "Optional[str]" = None) -> None:
+        """Add a named SQL query directly.
+
+        Args:
+            name: Name for the SQL query.
+            sql: Raw SQL content.
+            dialect: Optional dialect for the SQL statement.
+        """
+        if self._sql_loader is None:
+            from sqlspec.loader import SQLFileLoader
+
+            self._sql_loader = SQLFileLoader()
+
+        self._sql_loader.add_named_sql(name, sql, dialect)
+        logger.debug("Added named SQL: %s", name)
+
+    def get_sql(self, name: str) -> "SQL":
+        """Get a SQL object by name.
+
+        Args:
+            name: Name of the statement from SQL file comments.
+                  Hyphens in names are converted to underscores.
+
+        Returns:
+            SQL object ready for execution.
+        """
+        if self._sql_loader is None:
+            from sqlspec.loader import SQLFileLoader
+
+            self._sql_loader = SQLFileLoader()
+
+        return self._sql_loader.get_sql(name)
+
+    def list_sql_queries(self) -> "list[str]":
+        """List all available query names.
+
+        Returns:
+            Sorted list of query names.
+        """
+        if self._sql_loader is None:
+            return []
+        return self._sql_loader.list_queries()
+
+    def has_sql_query(self, name: str) -> bool:
+        """Check if a SQL query exists.
+
+        Args:
+            name: Query name to check.
+
+        Returns:
+            True if the query exists in the loader.
+        """
+        if self._sql_loader is None:
+            return False
+        return self._sql_loader.has_query(name)
+
+    def clear_sql_cache(self) -> None:
+        """Clear the SQL file cache."""
+        if self._sql_loader is not None:
+            self._sql_loader.clear_cache()
+            logger.debug("Cleared SQL cache")
+
+    def reload_sql_files(self) -> None:
+        """Reload all SQL files.
+
+        Note:
+            This clears the cache and requires calling load_sql_files again.
+        """
+        if self._sql_loader is not None:
+            self._sql_loader.clear_cache()
+            logger.debug("Cleared SQL cache for reload")
+
+    def get_sql_files(self) -> "list[str]":
+        """Get list of loaded SQL files.
+
+        Returns:
+            Sorted list of file paths.
+        """
+        if self._sql_loader is None:
+            return []
+        return self._sql_loader.list_files()
