@@ -1,5 +1,6 @@
 from sqlspec.extensions.litestar import handlers, providers
+from sqlspec.extensions.litestar.cli import database_group
 from sqlspec.extensions.litestar.config import DatabaseConfig
 from sqlspec.extensions.litestar.plugin import SQLSpec
 
-__all__ = ("DatabaseConfig", "SQLSpec", "handlers", "providers")
+__all__ = ("DatabaseConfig", "SQLSpec", "database_group", "handlers", "providers")
