@@ -25,7 +25,6 @@ if TYPE_CHECKING:
     from sqlspec.typing import ConnectionT, PoolT
 
 SESSION_TERMINUS_ASGI_EVENTS = {HTTP_RESPONSE_START, HTTP_DISCONNECT, WEBSOCKET_DISCONNECT, WEBSOCKET_CLOSE}
-"""ASGI events that terminate a session scope."""
 
 __all__ = (
     "SESSION_TERMINUS_ASGI_EVENTS",
@@ -39,7 +38,7 @@ __all__ = (
 
 
 def manual_handler_maker(connection_scope_key: str) -> "Callable[[Message, Scope], Coroutine[Any, Any, None]]":
-    """Set up the handler to close the connection.
+    """Create handler for manual connection management.
 
     Args:
         connection_scope_key: The key used to store the connection in the ASGI scope.
@@ -70,7 +69,7 @@ def autocommit_handler_maker(
     extra_commit_statuses: "Optional[set[int]]" = None,
     extra_rollback_statuses: "Optional[set[int]]" = None,
 ) -> "Callable[[Message, Scope], Coroutine[Any, Any, None]]":
-    """Set up the handler to issue a transaction commit or rollback based on response status codes.
+    """Create handler for automatic transaction commit/rollback based on response status.
 
     Args:
         connection_scope_key: The key used to store the connection in the ASGI scope.
@@ -125,27 +124,25 @@ def autocommit_handler_maker(
 def lifespan_handler_maker(
     config: "DatabaseConfigProtocol[Any, Any, Any]", pool_key: str
 ) -> "Callable[[Litestar], AbstractAsyncContextManager[None]]":
-    """Build the lifespan handler for managing the database connection pool.
-
-    The pool is created on application startup and closed on shutdown.
+    """Create lifespan handler for managing database connection pool lifecycle.
 
     Args:
         config: The database configuration object.
         pool_key: The key under which the connection pool will be stored in `app.state`.
 
     Returns:
-        The generated lifespan handler.
+        The lifespan handler function.
     """
 
     @contextlib.asynccontextmanager
     async def lifespan_handler(app: "Litestar") -> "AsyncGenerator[None, None]":
-        """Manages the database pool lifecycle.
+        """Manage database pool lifecycle for the application.
 
         Args:
             app: The Litestar application instance.
 
         Yields:
-            The generated lifespan handler.
+            Control to application during pool lifetime.
         """
         db_pool = await ensure_async_(config.create_pool)()
         app.state.update({pool_key: db_pool})
@@ -156,7 +153,7 @@ def lifespan_handler_maker(
             try:
                 await ensure_async_(config.close_pool)()
             except Exception as e:
-                if app.logger:  # pragma: no cover
+                if app.logger:
                     app.logger.warning("Error closing database pool for %s. Error: %s", pool_key, e)
 
     return lifespan_handler
@@ -165,35 +162,30 @@ def lifespan_handler_maker(
 def pool_provider_maker(
     config: "DatabaseConfigProtocol[ConnectionT, PoolT, DriverT]", pool_key: str
 ) -> "Callable[[State, Scope], Awaitable[PoolT]]":
-    """Build the pool provider to inject the application-level database pool.
+    """Create provider for injecting the application-level database pool.
 
     Args:
         config: The database configuration object.
         pool_key: The key used to store the connection pool in `app.state`.
 
     Returns:
-        The generated pool provider.
+        The pool provider function.
     """
 
     async def provide_pool(state: "State", scope: "Scope") -> "PoolT":
-        """Provides the database pool from `app.state`.
+        """Provide the database pool from application state.
 
         Args:
             state: The Litestar application State object.
             scope: The ASGI scope (unused for app-level pool).
 
-
         Returns:
             The database connection pool.
 
         Raises:
             ImproperConfigurationError: If the pool is not found in `app.state`.
         """
-        # The pool is stored in app.state by the lifespan handler.
-        # state.get(key) accesses app.state[key]
-        db_pool = state.get(pool_key)
-        if db_pool is None:
-            # This case should ideally not happen if the lifespan handler ran correctly.
+        if (db_pool := state.get(pool_key)) is None:
             msg = (
                 f"Database pool with key '{pool_key}' not found in application state. "
                 "Ensure the SQLSpec lifespan handler is correctly configured and has run."
@@ -207,9 +199,19 @@ def pool_provider_maker(
 def connection_provider_maker(
     config: "DatabaseConfigProtocol[ConnectionT, PoolT, DriverT]", pool_key: str, connection_key: str
 ) -> "Callable[[State, Scope], AsyncGenerator[ConnectionT, None]]":
+    """Create provider for database connections with proper lifecycle management.
+
+    Args:
+        config: The database configuration object.
+        pool_key: The key used to retrieve the connection pool from `app.state`.
+        connection_key: The key used to store the connection in the ASGI scope.
+
+    Returns:
+        The connection provider function.
+    """
+
     async def provide_connection(state: "State", scope: "Scope") -> "AsyncGenerator[ConnectionT, None]":
-        db_pool = state.get(pool_key)
-        if db_pool is None:
+        if (db_pool := state.get(pool_key)) is None:
             msg = f"Database pool with key '{pool_key}' not found. Cannot create a connection."
             raise ImproperConfigurationError(msg)
 
@@ -225,15 +227,13 @@ def connection_provider_maker(
             yield conn_instance
             return
 
-        entered_connection: Optional[ConnectionT] = None
+        entered_connection = await connection_cm.__aenter__()
         try:
-            entered_connection = await connection_cm.__aenter__()
             set_sqlspec_scope_state(scope, connection_key, entered_connection)
             yield entered_connection
         finally:
-            if entered_connection is not None:
-                await connection_cm.__aexit__(None, None, None)
-                delete_sqlspec_scope_state(scope, connection_key)  # Clear from scope
+            await connection_cm.__aexit__(None, None, None)
+            delete_sqlspec_scope_state(scope, connection_key)
 
     return provide_connection
 
@@ -241,6 +241,16 @@ def connection_provider_maker(
 def session_provider_maker(
     config: "DatabaseConfigProtocol[ConnectionT, PoolT, DriverT]", connection_dependency_key: str
 ) -> "Callable[[Any], AsyncGenerator[DriverT, None]]":
+    """Create provider for database driver sessions.
+
+    Args:
+        config: The database configuration object.
+        connection_dependency_key: The key used for connection dependency injection.
+
+    Returns:
+        The session provider function.
+    """
+
     async def provide_session(*args: Any, **kwargs: Any) -> "AsyncGenerator[DriverT, None]":
         yield cast("DriverT", config.driver_type(connection=args[0] if args else kwargs.get(connection_dependency_key)))  # pyright: ignore
 
