@@ -1,3 +1,9 @@
+"""Singleton pattern implementation for SQLSpec.
+
+Provides a thread-safe metaclass for implementing the singleton pattern.
+Used for creating classes that should have only one instance per class type.
+"""
+
 import threading
 from typing import Any, TypeVar
 
@@ -10,7 +16,6 @@ _T = TypeVar("_T")
 class SingletonMeta(type):
     """Metaclass for singleton pattern."""
 
-    # We store instances keyed by the class type
     _instances: dict[type, object] = {}
     _lock = threading.Lock()
 
