@@ -2,77 +2,50 @@
 
 from __future__ import annotations
 
-import tempfile
-from collections.abc import AsyncGenerator
-from pathlib import Path
+import math
 from typing import Any, Literal
 
-import pyarrow.parquet as pq
 import pytest
 
-from sqlspec.adapters.aiosqlite import AiosqliteConfig, AiosqliteDriver
-from sqlspec.statement.result import ArrowResult, SQLResult
-from sqlspec.statement.sql import SQL, SQLConfig
+from sqlspec.adapters.aiosqlite import AiosqliteDriver
+from sqlspec.core.result import SQLResult
+from sqlspec.core.statement import SQL
 
+pytestmark = pytest.mark.xdist_group("sqlite")
 ParamStyle = Literal["tuple_binds", "dict_binds", "named_binds"]
 
 
-@pytest.fixture
-async def aiosqlite_session() -> AsyncGenerator[AiosqliteDriver, None]:
-    """Create an aiosqlite session with test table."""
-    config = AiosqliteConfig(database=":memory:")
-
-    async with config.provide_session() as session:
-        # Create test table
-        await session.execute_script("""
-            CREATE TABLE IF NOT EXISTS test_table (
-                id INTEGER PRIMARY KEY AUTOINCREMENT,
-                name TEXT NOT NULL,
-                value INTEGER DEFAULT 0,
-                created_at DATETIME DEFAULT CURRENT_TIMESTAMP
-            )
-        """)
-        yield session
-        # Cleanup is automatic with in-memory database
-
-
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_basic_crud(aiosqlite_session: AiosqliteDriver) -> None:
     """Test basic CRUD operations."""
-    # INSERT
+
     insert_result = await aiosqlite_session.execute(
         "INSERT INTO test_table (name, value) VALUES (?, ?)", ("test_name", 42)
     )
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # SELECT
-    select_result = await aiosqlite_session.execute("SELECT name, value FROM test_table WHERE name = ?", ("test_name"))
+    select_result = await aiosqlite_session.execute("SELECT name, value FROM test_table WHERE name = ?", ("test_name",))
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
     assert len(select_result.data) == 1
     assert select_result.data[0]["name"] == "test_name"
     assert select_result.data[0]["value"] == 42
 
-    # UPDATE
     update_result = await aiosqlite_session.execute(
         "UPDATE test_table SET value = ? WHERE name = ?", (100, "test_name")
     )
     assert isinstance(update_result, SQLResult)
     assert update_result.rows_affected == 1
 
-    # Verify UPDATE
-    verify_result = await aiosqlite_session.execute("SELECT value FROM test_table WHERE name = ?", ("test_name"))
+    verify_result = await aiosqlite_session.execute("SELECT value FROM test_table WHERE name = ?", ("test_name",))
     assert isinstance(verify_result, SQLResult)
     assert verify_result.data is not None
     assert verify_result.data[0]["value"] == 100
 
-    # DELETE
-    delete_result = await aiosqlite_session.execute("DELETE FROM test_table WHERE name = ?", ("test_name"))
+    delete_result = await aiosqlite_session.execute("DELETE FROM test_table WHERE name = ?", ("test_name",))
     assert isinstance(delete_result, SQLResult)
     assert delete_result.rows_affected == 1
 
-    # Verify DELETE
     empty_result = await aiosqlite_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(empty_result, SQLResult)
     assert empty_result.data is not None
@@ -80,47 +53,51 @@ async def test_aiosqlite_basic_crud(aiosqlite_session: AiosqliteDriver) -> None:
 
 
 @pytest.mark.parametrize(
-    ("params", "style"),
+    ("parameters", "style"),
     [
-        pytest.param(("test_value"), "tuple_binds", id="tuple_binds"),
+        pytest.param(("test_value",), "tuple_binds", id="tuple_binds"),
         pytest.param({"name": "test_value"}, "dict_binds", id="dict_binds"),
     ],
 )
-@pytest.mark.xdist_group("aiosqlite")
-async def test_aiosqlite_parameter_styles(aiosqlite_session: AiosqliteDriver, params: Any, style: ParamStyle) -> None:
+async def test_aiosqlite_parameter_styles(
+    aiosqlite_session: AiosqliteDriver, parameters: Any, style: ParamStyle
+) -> None:
     """Test different parameter binding styles."""
-    # Insert test data
-    await aiosqlite_session.execute("INSERT INTO test_table (name) VALUES (?)", ("test_value"))
 
-    # Test parameter style
+    await aiosqlite_session.execute("DELETE FROM test_table")
+    await aiosqlite_session.commit()
+
+    await aiosqlite_session.execute("INSERT INTO test_table (name) VALUES (?)", ("test_value",))
+
     if style == "tuple_binds":
         sql = "SELECT name FROM test_table WHERE name = ?"
-    else:  # dict_binds
+    else:
         sql = "SELECT name FROM test_table WHERE name = :name"
 
-    result = await aiosqlite_session.execute(sql, params)
+    result = await aiosqlite_session.execute(sql, parameters)
     assert isinstance(result, SQLResult)
     assert result.data is not None
     assert len(result.data) == 1
     assert result.data[0]["name"] == "test_value"
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_execute_many(aiosqlite_session: AiosqliteDriver) -> None:
     """Test execute_many functionality."""
-    params_list = [("name1", 1), ("name2", 2), ("name3", 3)]
 
-    result = await aiosqlite_session.execute_many("INSERT INTO test_table (name, value) VALUES (?, ?)", params_list)
+    await aiosqlite_session.execute("DELETE FROM test_table")
+    await aiosqlite_session.commit()
+
+    parameters_list = [("name1", 1), ("name2", 2), ("name3", 3)]
+
+    result = await aiosqlite_session.execute_many("INSERT INTO test_table (name, value) VALUES (?, ?)", parameters_list)
     assert isinstance(result, SQLResult)
-    assert result.rows_affected == len(params_list)
+    assert result.rows_affected == len(parameters_list)
 
-    # Verify all records were inserted
     select_result = await aiosqlite_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
-    assert select_result.data[0]["count"] == len(params_list)
+    assert select_result.data[0]["count"] == len(parameters_list)
 
-    # Verify data integrity
     ordered_result = await aiosqlite_session.execute("SELECT name, value FROM test_table ORDER BY name")
     assert isinstance(ordered_result, SQLResult)
     assert ordered_result.data is not None
@@ -129,7 +106,6 @@ async def test_aiosqlite_execute_many(aiosqlite_session: AiosqliteDriver) -> Non
     assert ordered_result.data[0]["value"] == 1
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_execute_script(aiosqlite_session: AiosqliteDriver) -> None:
     """Test execute_script functionality."""
     script = """
@@ -139,11 +115,10 @@ async def test_aiosqlite_execute_script(aiosqlite_session: AiosqliteDriver) -> N
     """
 
     result = await aiosqlite_session.execute_script(script)
-    # Script execution now returns SQLResult object
+
     assert isinstance(result, SQLResult)
     assert result.operation_type == "SCRIPT"
 
-    # Verify script effects
     select_result = await aiosqlite_session.execute(
         "SELECT name, value FROM test_table WHERE name LIKE 'script_test%' ORDER BY name"
     )
@@ -156,57 +131,50 @@ async def test_aiosqlite_execute_script(aiosqlite_session: AiosqliteDriver) -> N
     assert select_result.data[1]["value"] == 888
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_result_methods(aiosqlite_session: AiosqliteDriver) -> None:
     """Test SelectResult and ExecuteResult methods."""
-    # Insert test data
+
+    await aiosqlite_session.execute("DELETE FROM test_table")
+    await aiosqlite_session.commit()
+
     await aiosqlite_session.execute_many(
         "INSERT INTO test_table (name, value) VALUES (?, ?)", [("result1", 10), ("result2", 20), ("result3", 30)]
     )
 
-    # Test SelectResult methods
     result = await aiosqlite_session.execute("SELECT * FROM test_table ORDER BY name")
     assert isinstance(result, SQLResult)
 
-    # Test get_first()
     first_row = result.get_first()
     assert first_row is not None
     assert first_row["name"] == "result1"
 
-    # Test get_count()
     assert result.get_count() == 3
 
-    # Test is_empty()
     assert not result.is_empty()
 
-    # Test empty result
-    empty_result = await aiosqlite_session.execute("SELECT * FROM test_table WHERE name = ?", ("nonexistent"))
+    empty_result = await aiosqlite_session.execute("SELECT * FROM test_table WHERE name = ?", ("nonexistent",))
     assert isinstance(empty_result, SQLResult)
     assert empty_result.is_empty()
     assert empty_result.get_first() is None
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_error_handling(aiosqlite_session: AiosqliteDriver) -> None:
     """Test error handling and exception propagation."""
-    # Test invalid SQL
-    with pytest.raises(Exception):  # aiosqlite.OperationalError
+
+    with pytest.raises(Exception):
         await aiosqlite_session.execute("INVALID SQL STATEMENT")
 
-    # Test constraint violation
     await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("unique_test", 1))
 
-    # Try to insert with invalid column reference
-    with pytest.raises(Exception):  # aiosqlite.OperationalError
+    with pytest.raises(Exception):
         await aiosqlite_session.execute("SELECT nonexistent_column FROM test_table")
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_data_types(aiosqlite_session: AiosqliteDriver) -> None:
     """Test SQLite data type handling with aiosqlite."""
-    # Create table with various data types
+
     await aiosqlite_session.execute_script("""
-        CREATE TABLE data_types_test (
+        CREATE TABLE data_types_test_unique (
             id INTEGER PRIMARY KEY,
             text_col TEXT,
             integer_col INTEGER,
@@ -216,21 +184,17 @@ async def test_aiosqlite_data_types(aiosqlite_session: AiosqliteDriver) -> None:
         )
     """)
 
-    # Insert data with various types
-    import math
-
     test_data = ("text_value", 42, math.pi, b"binary_data", None)
 
     insert_result = await aiosqlite_session.execute(
-        "INSERT INTO data_types_test (text_col, integer_col, real_col, blob_col, null_col) VALUES (?, ?, ?, ?, ?)",
+        "INSERT INTO data_types_test_unique (text_col, integer_col, real_col, blob_col, null_col) VALUES (?, ?, ?, ?, ?)",
         test_data,
     )
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Retrieve and verify data
     select_result = await aiosqlite_session.execute(
-        "SELECT text_col, integer_col, real_col, blob_col, null_col FROM data_types_test"
+        "SELECT text_col, integer_col, real_col, blob_col, null_col FROM data_types_test_unique"
     )
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
@@ -243,34 +207,32 @@ async def test_aiosqlite_data_types(aiosqlite_session: AiosqliteDriver) -> None:
     assert row["blob_col"] == b"binary_data"
     assert row["null_col"] is None
 
-    # Clean up
-    await aiosqlite_session.execute_script("DROP TABLE data_types_test")
+    await aiosqlite_session.execute_script("DROP TABLE data_types_test_unique")
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_transactions(aiosqlite_session: AiosqliteDriver) -> None:
     """Test transaction behavior."""
-    # SQLite auto-commit mode test
+
     await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("transaction_test", 100))
 
-    # Verify data is committed
     result = await aiosqlite_session.execute(
-        "SELECT COUNT(*) as count FROM test_table WHERE name = ?", ("transaction_test")
+        "SELECT COUNT(*) as count FROM test_table WHERE name = ?", ("transaction_test",)
     )
     assert isinstance(result, SQLResult)
     assert result.data is not None
     assert result.data[0]["count"] == 1
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_complex_queries(aiosqlite_session: AiosqliteDriver) -> None:
     """Test complex SQL queries."""
-    # Insert test data
+
+    await aiosqlite_session.execute("DELETE FROM test_table")
+    await aiosqlite_session.commit()
+
     test_data = [("Alice", 25), ("Bob", 30), ("Charlie", 35), ("Diana", 28)]
 
     await aiosqlite_session.execute_many("INSERT INTO test_table (name, value) VALUES (?, ?)", test_data)
 
-    # Test JOIN (self-join)
     join_result = await aiosqlite_session.execute("""
         SELECT t1.name as name1, t2.name as name2, t1.value as value1, t2.value as value2
         FROM test_table t1
@@ -283,7 +245,6 @@ async def test_aiosqlite_complex_queries(aiosqlite_session: AiosqliteDriver) ->
     assert join_result.data is not None
     assert len(join_result.data) == 3
 
-    # Test aggregation
     agg_result = await aiosqlite_session.execute("""
         SELECT
             COUNT(*) as total_count,
@@ -299,7 +260,6 @@ async def test_aiosqlite_complex_queries(aiosqlite_session: AiosqliteDriver) ->
     assert agg_result.data[0]["min_value"] == 25
     assert agg_result.data[0]["max_value"] == 35
 
-    # Test subquery
     subquery_result = await aiosqlite_session.execute("""
         SELECT name, value
         FROM test_table
@@ -308,15 +268,14 @@ async def test_aiosqlite_complex_queries(aiosqlite_session: AiosqliteDriver) ->
     """)
     assert isinstance(subquery_result, SQLResult)
     assert subquery_result.data is not None
-    assert len(subquery_result.data) == 2  # Bob and Charlie
+    assert len(subquery_result.data) == 2
     assert subquery_result.data[0]["name"] == "Bob"
     assert subquery_result.data[1]["name"] == "Charlie"
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_schema_operations(aiosqlite_session: AiosqliteDriver) -> None:
     """Test schema operations (DDL)."""
-    # Create a new table
+
     create_result = await aiosqlite_session.execute_script("""
         CREATE TABLE schema_test (
             id INTEGER PRIMARY KEY,
@@ -327,41 +286,35 @@ async def test_aiosqlite_schema_operations(aiosqlite_session: AiosqliteDriver) -
     assert isinstance(create_result, SQLResult)
     assert create_result.operation_type == "SCRIPT"
 
-    # Insert data into new table
     insert_result = await aiosqlite_session.execute(
-        "INSERT INTO schema_test (description) VALUES (?)", ("test description")
+        "INSERT INTO schema_test (description) VALUES (?)", ("test description",)
     )
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Verify table structure
     pragma_result = await aiosqlite_session.execute("PRAGMA table_info(schema_test)")
     assert isinstance(pragma_result, SQLResult)
     assert pragma_result.data is not None
-    assert len(pragma_result.data) == 3  # id, description, created_at
+    assert len(pragma_result.data) == 3
 
-    # Drop table
     drop_result = await aiosqlite_session.execute_script("DROP TABLE schema_test")
     assert isinstance(drop_result, SQLResult)
     assert drop_result.operation_type == "SCRIPT"
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_column_names_and_metadata(aiosqlite_session: AiosqliteDriver) -> None:
     """Test column names and result metadata."""
-    # Insert test data
+
     await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("metadata_test", 123))
 
-    # Test column names
     result = await aiosqlite_session.execute(
-        "SELECT id, name, value, created_at FROM test_table WHERE name = ?", ("metadata_test")
+        "SELECT id, name, value, created_at FROM test_table WHERE name = ?", ("metadata_test",)
     )
     assert isinstance(result, SQLResult)
     assert result.column_names == ["id", "name", "value", "created_at"]
     assert result.data is not None
     assert len(result.data) == 1
 
-    # Test that we can access data by column name
     row = result.data[0]
     assert row["name"] == "metadata_test"
     assert row["value"] == 123
@@ -369,45 +322,15 @@ async def test_aiosqlite_column_names_and_metadata(aiosqlite_session: AiosqliteD
     assert row["created_at"] is not None
 
 
-@pytest.mark.xdist_group("aiosqlite")
-async def test_aiosqlite_with_schema_type(aiosqlite_session: AiosqliteDriver) -> None:
-    """Test aiosqlite driver with schema type conversion."""
-    from dataclasses import dataclass
-
-    @dataclass
-    class TestRecord:
-        id: int | None
-        name: str
-        value: int
-
-    # Insert test data
-    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("schema_test", 456))
-
-    # Query with schema type
-    result = await aiosqlite_session.execute(
-        "SELECT id, name, value FROM test_table WHERE name = ?", ("schema_test"), schema_type=TestRecord
-    )
-
-    assert isinstance(result, SQLResult)
-    assert result.data is not None
-    assert len(result.data) == 1
-
-    # The data should be converted to the schema type by the ResultConverter
-    assert result.column_names == ["id", "name", "value"]
-
-
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_performance_bulk_operations(aiosqlite_session: AiosqliteDriver) -> None:
     """Test performance with bulk operations."""
-    # Generate bulk data
+
     bulk_data = [(f"bulk_user_{i}", i * 10) for i in range(100)]
 
-    # Bulk insert
     result = await aiosqlite_session.execute_many("INSERT INTO test_table (name, value) VALUES (?, ?)", bulk_data)
     assert isinstance(result, SQLResult)
     assert result.rows_affected == 100
 
-    # Bulk select
     select_result = await aiosqlite_session.execute(
         "SELECT COUNT(*) as count FROM test_table WHERE name LIKE 'bulk_user_%'"
     )
@@ -415,7 +338,6 @@ async def test_aiosqlite_performance_bulk_operations(aiosqlite_session: Aiosqlit
     assert select_result.data is not None
     assert select_result.data[0]["count"] == 100
 
-    # Test pagination-like query
     page_result = await aiosqlite_session.execute(
         "SELECT name, value FROM test_table WHERE name LIKE 'bulk_user_%' ORDER BY value LIMIT 10 OFFSET 20"
     )
@@ -425,35 +347,36 @@ async def test_aiosqlite_performance_bulk_operations(aiosqlite_session: Aiosqlit
     assert page_result.data[0]["name"] == "bulk_user_20"
 
 
-@pytest.mark.xdist_group("aiosqlite")
 async def test_aiosqlite_sqlite_specific_features(aiosqlite_session: AiosqliteDriver) -> None:
     """Test SQLite-specific features with aiosqlite."""
-    # Test PRAGMA statements
+
     pragma_result = await aiosqlite_session.execute("PRAGMA user_version")
     assert isinstance(pragma_result, SQLResult)
     assert pragma_result.data is not None
 
-    # Test SQLite functions
     sqlite_result = await aiosqlite_session.execute("SELECT sqlite_version() as version")
     assert isinstance(sqlite_result, SQLResult)
     assert sqlite_result.data is not None
     assert sqlite_result.data[0]["version"] is not None
 
-    # Test JSON operations (if JSON1 extension is available)
     try:
         json_result = await aiosqlite_session.execute("SELECT json('{}') as json_test")
         assert isinstance(json_result, SQLResult)
         assert json_result.data is not None
     except Exception:
-        # JSON1 extension might not be available
         pass
-    non_strict_config = SQLConfig(enable_parsing=False, enable_validation=False)
 
-    await aiosqlite_session.execute("ATTACH DATABASE ':memory:' AS temp_db", _config=non_strict_config)
+    from sqlspec.core.statement import StatementConfig
+
+    non_strict_config = StatementConfig(enable_parsing=False, enable_validation=False)
+
+    await aiosqlite_session.execute("ATTACH DATABASE ':memory:' AS temp_db", statement_config=non_strict_config)
     await aiosqlite_session.execute(
-        "CREATE TABLE temp_db.temp_table (id INTEGER, name TEXT)", _config=non_strict_config
+        "CREATE TABLE temp_db.temp_table (id INTEGER, name TEXT)", statement_config=non_strict_config
+    )
+    await aiosqlite_session.execute(
+        "INSERT INTO temp_db.temp_table VALUES (1, 'temp')", statement_config=non_strict_config
     )
-    await aiosqlite_session.execute("INSERT INTO temp_db.temp_table VALUES (1, 'temp')", _config=non_strict_config)
 
     temp_result = await aiosqlite_session.execute("SELECT * FROM temp_db.temp_table")
     assert isinstance(temp_result, SQLResult)
@@ -462,39 +385,135 @@ async def test_aiosqlite_sqlite_specific_features(aiosqlite_session: AiosqliteDr
     assert temp_result.data[0]["name"] == "temp"
 
     try:
-        await aiosqlite_session.execute("DETACH DATABASE temp_db", _config=non_strict_config)
+        await aiosqlite_session.execute("DETACH DATABASE temp_db", statement_config=non_strict_config)
     except Exception:
-        # Database might be locked, which is fine for this test
         pass
 
 
-@pytest.mark.asyncio
-async def test_aiosqlite_fetch_arrow_table(aiosqlite_session: AiosqliteDriver) -> None:
-    """Integration test: fetch_arrow_table returns ArrowResult with correct pyarrow.Table."""
-    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("arrow1", 111))
-    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("arrow2", 222))
-    statement = SQL("SELECT name, value FROM test_table ORDER BY name")
-    result = await aiosqlite_session.fetch_arrow_table(statement)
-    assert isinstance(result, ArrowResult)
-    assert result.num_rows == 2
-    assert set(result.column_names) == {"name", "value"}
+async def test_aiosqlite_sql_object_integration(aiosqlite_session: AiosqliteDriver) -> None:
+    """Test integration with SQL object."""
+
+    sql_obj = SQL("SELECT name, value FROM test_table WHERE value > ?")
+
+    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("sql_test", 50))
+
+    result = await aiosqlite_session.execute(sql_obj, (25,))
+    assert isinstance(result, SQLResult)
     assert result.data is not None
-    table = result.data
-    names = table["name"].to_pylist()
-    assert "arrow1" in names and "arrow2" in names
+    assert len(result.data) == 1
+    assert result.data[0]["name"] == "sql_test"
+    assert result.data[0]["value"] == 50
 
 
-@pytest.mark.asyncio
-async def test_aiosqlite_to_parquet(aiosqlite_session: AiosqliteDriver) -> None:
-    """Integration test: to_parquet writes correct data to a Parquet file."""
-    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("pq1", 123))
-    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("pq2", 456))
-    statement = SQL("SELECT name, value FROM test_table ORDER BY name")
-    with tempfile.TemporaryDirectory() as tmpdir:
-        output_path = Path(tmpdir) / "partitioned_data"
-        await aiosqlite_session.export_to_storage(statement, destination_uri=str(output_path), format="parquet")
-        table = pq.read_table(f"{output_path}.parquet")
-        assert table.num_rows == 2
-        assert set(table.column_names) == {"name", "value"}
-        names = table.column("name").to_pylist()
-        assert "pq1" in names and "pq2" in names
+async def test_aiosqlite_core_result_features(aiosqlite_session: AiosqliteDriver) -> None:
+    """Test SQLResult features."""
+
+    test_data = [("core1", 10), ("core2", 20), ("core3", 30)]
+    await aiosqlite_session.execute_many("INSERT INTO test_table (name, value) VALUES (?, ?)", test_data)
+
+    result = await aiosqlite_session.execute("SELECT * FROM test_table WHERE name LIKE 'core%' ORDER BY name")
+    assert isinstance(result, SQLResult)
+
+    assert result.get_count() == 3
+    assert not result.is_empty()
+
+    first = result.get_first()
+    assert first is not None
+    assert first["name"] == "core1"
+
+    assert "name" in result.column_names
+    assert "value" in result.column_names
+
+    assert len(result.data) == 3
+    assert all(row["name"].startswith("core") for row in result.data)
+
+
+async def test_aiosqlite_for_update_generates_sql(aiosqlite_session: AiosqliteDriver) -> None:
+    """Test that FOR UPDATE generates SQL for aiosqlite (though SQLite doesn't support row-level locking)."""
+    from sqlspec import sql
+
+    # Create test table
+    await aiosqlite_session.execute_script("""
+        DROP TABLE IF EXISTS test_table;
+        CREATE TABLE test_table (
+            id INTEGER PRIMARY KEY,
+            name TEXT,
+            value INTEGER
+        );
+    """)
+
+    # Insert test data
+    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("aiosqlite_test", 100))
+
+    # Should generate SQL even though SQLite doesn't support the functionality
+    query = sql.select("*").from_("test_table").where_eq("name", "aiosqlite_test").for_update()
+    stmt = query.build()
+    # SQLite doesn't support FOR UPDATE, so SQLGlot strips it out (expected behavior)
+    assert "FOR UPDATE" not in stmt.sql
+    assert "SELECT" in stmt.sql  # But the rest of the query works
+
+    # Should execute without error (SQLite just ignores the FOR UPDATE)
+    result = await aiosqlite_session.execute(query)
+    assert result is not None
+    assert len(result.data) == 1
+    assert result.data[0]["name"] == "aiosqlite_test"
+
+
+async def test_aiosqlite_for_share_generates_sql_but_may_not_work(aiosqlite_session: AiosqliteDriver) -> None:
+    """Test that FOR SHARE generates SQL for aiosqlite but note it doesn't provide row-level locking."""
+    from sqlspec import sql
+
+    # Create test table
+    await aiosqlite_session.execute_script("""
+        DROP TABLE IF EXISTS test_table;
+        CREATE TABLE test_table (
+            id INTEGER PRIMARY KEY,
+            name TEXT,
+            value INTEGER
+        );
+    """)
+
+    # Insert test data
+    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("aiosqlite_share", 200))
+
+    # Should generate SQL even though SQLite doesn't support the functionality
+    query = sql.select("*").from_("test_table").where_eq("name", "aiosqlite_share").for_share()
+    stmt = query.build()
+    # SQLite doesn't support FOR SHARE, so SQLGlot strips it out (expected behavior)
+    assert "FOR SHARE" not in stmt.sql
+    assert "SELECT" in stmt.sql  # But the rest of the query works
+
+    # Should execute without error (SQLite just ignores the FOR SHARE)
+    result = await aiosqlite_session.execute(query)
+    assert result is not None
+    assert len(result.data) == 1
+    assert result.data[0]["name"] == "aiosqlite_share"
+
+
+async def test_aiosqlite_for_update_skip_locked_generates_sql(aiosqlite_session: AiosqliteDriver) -> None:
+    """Test that FOR UPDATE SKIP LOCKED generates SQL for aiosqlite."""
+    from sqlspec import sql
+
+    # Create test table
+    await aiosqlite_session.execute_script("""
+        DROP TABLE IF EXISTS test_table;
+        CREATE TABLE test_table (
+            id INTEGER PRIMARY KEY,
+            name TEXT,
+            value INTEGER
+        );
+    """)
+
+    # Insert test data
+    await aiosqlite_session.execute("INSERT INTO test_table (name, value) VALUES (?, ?)", ("aiosqlite_skip", 300))
+
+    # Should generate SQL even though SQLite doesn't support the functionality
+    query = sql.select("*").from_("test_table").where_eq("name", "aiosqlite_skip").for_update(skip_locked=True)
+    stmt = query.build()
+    # The exact SQL generated may vary based on dialect support
+    assert stmt.sql is not None
+
+    # Should execute (SQLite will ignore unsupported clauses)
+    result = await aiosqlite_session.execute(query)
+    assert result is not None
+    assert len(result.data) == 1
