@@ -1,19 +1,14 @@
 """Test DuckDB driver implementation."""
 
-from __future__ import annotations
-
-import tempfile
 from collections.abc import Generator
-from pathlib import Path
 from typing import Any, Literal
 
-import pyarrow as pa
-import pyarrow.parquet as pq
 import pytest
 
 from sqlspec.adapters.duckdb import DuckDBConfig, DuckDBDriver
-from sqlspec.statement.result import ArrowResult, SQLResult
-from sqlspec.statement.sql import SQL
+from sqlspec.core.result import SQLResult
+
+pytestmark = pytest.mark.xdist_group("duckdb")
 
 ParamStyle = Literal["tuple_binds", "dict_binds"]
 
@@ -25,42 +20,43 @@ def duckdb_session() -> Generator[DuckDBDriver, None, None]:
     Returns:
         A DuckDB session with a test table.
     """
-    adapter = DuckDBConfig()
-    with adapter.provide_session() as session:
-        session.execute_script("CREATE SEQUENCE IF NOT EXISTS test_id_seq START 1")
-        create_table_sql = """
-            CREATE TABLE IF NOT EXISTS test_table (
-                id INTEGER PRIMARY KEY DEFAULT nextval('test_id_seq'),
-                name TEXT NOT NULL
-            )
-        """
-        session.execute_script(create_table_sql)
-        yield session
-        # Clean up
-        session.execute_script("DROP TABLE IF EXISTS test_table")
-        session.execute_script("DROP SEQUENCE IF EXISTS test_id_seq")
+    adapter = DuckDBConfig(pool_config={"database": ":memory:"})
+    try:
+        with adapter.provide_session() as session:
+            session.execute_script("CREATE SEQUENCE IF NOT EXISTS test_id_seq START 1")
+            create_table_sql = """
+                CREATE TABLE IF NOT EXISTS test_table (
+                    id INTEGER PRIMARY KEY DEFAULT nextval('test_id_seq'),
+                    name TEXT NOT NULL
+                )
+            """
+            session.execute_script(create_table_sql)
+            yield session
+
+            session.execute_script("DROP TABLE IF EXISTS test_table")
+            session.execute_script("DROP SEQUENCE IF EXISTS test_id_seq")
+    finally:
+        adapter.close_pool()
 
 
 @pytest.mark.parametrize(
-    ("params", "style"),
+    ("parameters", "style"),
     [
         pytest.param(("test_name", 1), "tuple_binds", id="tuple_binds"),
         pytest.param({"name": "test_name", "id": 1}, "dict_binds", id="dict_binds"),
     ],
 )
-@pytest.mark.xdist_group("duckdb")
-def test_insert(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) -> None:
+def test_insert(duckdb_session: DuckDBDriver, parameters: Any, style: ParamStyle) -> None:
     """Test inserting data with different parameter styles."""
     if style == "tuple_binds":
         sql = "INSERT INTO test_table (name, id) VALUES (?, ?)"
     else:
         sql = "INSERT INTO test_table (name, id) VALUES (:name, :id)"
 
-    result = duckdb_session.execute(sql, params)
+    result = duckdb_session.execute(sql, parameters)
     assert isinstance(result, SQLResult)
     assert result.rows_affected == 1
 
-    # Verify insertion
     select_result = duckdb_session.execute("SELECT name, id FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
@@ -72,26 +68,24 @@ def test_insert(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) ->
 
 
 @pytest.mark.parametrize(
-    ("params", "style"),
+    ("parameters", "style"),
     [
         pytest.param(("test_name", 1), "tuple_binds", id="tuple_binds"),
         pytest.param({"name": "test_name", "id": 1}, "dict_binds", id="dict_binds"),
     ],
 )
-@pytest.mark.xdist_group("duckdb")
-def test_select(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) -> None:
+def test_select(duckdb_session: DuckDBDriver, parameters: Any, style: ParamStyle) -> None:
     """Test selecting data with different parameter styles."""
-    # Insert test record
+
     if style == "tuple_binds":
         insert_sql = "INSERT INTO test_table (name, id) VALUES (?, ?)"
     else:
         insert_sql = "INSERT INTO test_table (name, id) VALUES (:name, :id)"
 
-    insert_result = duckdb_session.execute(insert_sql, params)
+    insert_result = duckdb_session.execute(insert_sql, parameters)
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Test select
     select_result = duckdb_session.execute("SELECT name, id FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
@@ -99,15 +93,14 @@ def test_select(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) ->
     assert select_result.data[0]["name"] == "test_name"
     assert select_result.data[0]["id"] == 1
 
-    # Test select with a WHERE clause
     if style == "tuple_binds":
         select_where_sql = "SELECT id FROM test_table WHERE name = ?"
-        where_params = "test_name"
+        where_parameters = "test_name"
     else:
         select_where_sql = "SELECT id FROM test_table WHERE name = :name"
-        where_params = {"name": "test_name"}
+        where_parameters = {"name": "test_name"}
 
-    where_result = duckdb_session.execute(select_where_sql, where_params)
+    where_result = duckdb_session.execute(select_where_sql, where_parameters)
     assert isinstance(where_result, SQLResult)
     assert where_result.data is not None
     assert len(where_result.data) == 1
@@ -117,103 +110,58 @@ def test_select(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) ->
 
 
 @pytest.mark.parametrize(
-    ("params", "style"),
+    ("parameters", "style"),
     [
         pytest.param(("test_name", 1), "tuple_binds", id="tuple_binds"),
         pytest.param({"name": "test_name", "id": 1}, "dict_binds", id="dict_binds"),
     ],
 )
-@pytest.mark.xdist_group("duckdb")
-def test_select_value(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) -> None:
+def test_select_value(duckdb_session: DuckDBDriver, parameters: Any, style: ParamStyle) -> None:
     """Test select value with different parameter styles."""
-    # Insert test record
+
     if style == "tuple_binds":
         insert_sql = "INSERT INTO test_table (name, id) VALUES (?, ?)"
     else:
         insert_sql = "INSERT INTO test_table (name, id) VALUES (:name, :id)"
 
-    insert_result = duckdb_session.execute(insert_sql, params)
+    insert_result = duckdb_session.execute(insert_sql, parameters)
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Test select value
     if style == "tuple_binds":
         value_sql = "SELECT name FROM test_table WHERE id = ?"
-        value_params = 1
+        value_parameters = 1
     else:
         value_sql = "SELECT name FROM test_table WHERE id = :id"
-        value_params = {"id": 1}
+        value_parameters = {"id": 1}
 
-    value_result = duckdb_session.execute(value_sql, value_params)
+    value_result = duckdb_session.execute(value_sql, value_parameters)
     assert isinstance(value_result, SQLResult)
     assert value_result.data is not None
     assert len(value_result.data) == 1
     assert value_result.column_names is not None
 
-    # Extract single value using column name
     value = value_result.data[0][value_result.column_names[0]]
     assert value == "test_name"
 
     duckdb_session.execute_script("DELETE FROM test_table")
 
 
-@pytest.mark.parametrize(
-    ("params", "style"),
-    [
-        pytest.param(("arrow_name", 1), "tuple_binds", id="tuple_binds"),
-        pytest.param({"name": "arrow_name", "id": 1}, "dict_binds", id="dict_binds"),
-    ],
-)
-@pytest.mark.xdist_group("duckdb")
-def test_select_arrow(duckdb_session: DuckDBDriver, params: Any, style: ParamStyle) -> None:
-    """Test selecting data as an Arrow Table."""
-    # Insert test record
-    if style == "tuple_binds":
-        insert_sql = "INSERT INTO test_table (name, id) VALUES (?, ?)"
-    else:
-        insert_sql = "INSERT INTO test_table (name, id) VALUES (:name, :id)"
-
-    insert_result = duckdb_session.execute(insert_sql, params)
-    assert isinstance(insert_result, SQLResult)
-    assert insert_result.rows_affected == 1
-
-    # Test select_arrow using mixins
-    if hasattr(duckdb_session, "fetch_arrow_table"):
-        select_sql = "SELECT name, id FROM test_table WHERE id = 1"
-        arrow_result = duckdb_session.fetch_arrow_table(select_sql)
-
-        assert isinstance(arrow_result, ArrowResult)
-        arrow_table = arrow_result.data
-        assert isinstance(arrow_table, pa.Table)
-        assert arrow_table.num_rows == 1
-        assert arrow_table.num_columns == 2
-        assert arrow_table.column_names == ["name", "id"]
-        assert arrow_table.column("name").to_pylist() == ["arrow_name"]
-        assert arrow_table.column("id").to_pylist() == [1]
-    else:
-        pytest.skip("DuckDB driver does not support Arrow operations")
-
-    duckdb_session.execute_script("DELETE FROM test_table")
-
-
-@pytest.mark.xdist_group("duckdb")
 def test_execute_many_insert(duckdb_session: DuckDBDriver) -> None:
     """Test execute_many functionality for batch inserts."""
     insert_sql = "INSERT INTO test_table (name, id) VALUES (?, ?)"
-    params_list = [("name1", 10), ("name2", 20), ("name3", 30)]
+    parameters_list = [("name1", 10), ("name2", 20), ("name3", 30)]
 
-    result = duckdb_session.execute_many(insert_sql, params_list)
+    result = duckdb_session.execute_many(insert_sql, parameters_list)
     assert isinstance(result, SQLResult)
-    assert result.rows_affected == len(params_list)
+    assert result.rows_affected == len(parameters_list)
 
-    # Verify all records were inserted
     select_result = duckdb_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
-    assert select_result.data[0]["count"] == len(params_list)
+    assert select_result.data[0]["count"] == len(parameters_list)
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_execute_script(duckdb_session: DuckDBDriver) -> None:
     """Test execute_script functionality for multi-statement scripts."""
     script = """
@@ -224,57 +172,49 @@ def test_execute_script(duckdb_session: DuckDBDriver) -> None:
     result = duckdb_session.execute_script(script)
     assert isinstance(result, SQLResult)
 
-    # Verify script executed successfully
     select_result = duckdb_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
     assert select_result.data[0]["count"] == 2
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_update_operation(duckdb_session: DuckDBDriver) -> None:
     """Test UPDATE operations."""
-    # Insert a record first
+
     insert_result = duckdb_session.execute("INSERT INTO test_table (name, id) VALUES (?, ?)", ("original_name", 42))
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Update the record
     update_result = duckdb_session.execute("UPDATE test_table SET name = ? WHERE id = ?", ("updated_name", 42))
     assert isinstance(update_result, SQLResult)
     assert update_result.rows_affected == 1
 
-    # Verify the update
     select_result = duckdb_session.execute("SELECT name FROM test_table WHERE id = ?", (42))
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
     assert select_result.data[0]["name"] == "updated_name"
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_delete_operation(duckdb_session: DuckDBDriver) -> None:
     """Test DELETE operations."""
-    # Insert a record first
+
     insert_result = duckdb_session.execute("INSERT INTO test_table (name, id) VALUES (?, ?)", ("to_delete", 99))
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Delete the record
     delete_result = duckdb_session.execute("DELETE FROM test_table WHERE id = ?", (99))
     assert isinstance(delete_result, SQLResult)
     assert delete_result.rows_affected == 1
 
-    # Verify the deletion
     select_result = duckdb_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
     assert select_result.data[0]["count"] == 0
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_data_types(duckdb_session: DuckDBDriver) -> None:
     """Test DuckDB-specific data types and functionality."""
-    # Create table with various DuckDB data types
+
     duckdb_session.execute_script("""
         CREATE TABLE data_types_test (
             id INTEGER,
@@ -288,7 +228,6 @@ def test_duckdb_data_types(duckdb_session: DuckDBDriver) -> None:
         )
     """)
 
-    # Insert test data with DuckDB-specific types
     insert_sql = """
         INSERT INTO data_types_test VALUES (
             1,
@@ -304,7 +243,6 @@ def test_duckdb_data_types(duckdb_session: DuckDBDriver) -> None:
     result = duckdb_session.execute(insert_sql)
     assert result.rows_affected == 1
 
-    # Query and verify data types
     select_result = duckdb_session.execute("SELECT * FROM data_types_test")
     assert len(select_result.data) == 1
     row = select_result.data[0]
@@ -312,18 +250,16 @@ def test_duckdb_data_types(duckdb_session: DuckDBDriver) -> None:
     assert row["id"] == 1
     assert row["text_col"] == "test_text"
     assert row["boolean_col"] is True
-    # Array and JSON handling may vary based on DuckDB version
+
     assert row["array_col"] is not None
     assert row["json_col"] is not None
 
-    # Clean up
     duckdb_session.execute_script("DROP TABLE data_types_test")
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_complex_queries(duckdb_session: DuckDBDriver) -> None:
     """Test complex SQL queries with DuckDB."""
-    # Create additional tables for complex queries
+
     duckdb_session.execute_script("""
         CREATE TABLE departments (
             dept_id INTEGER PRIMARY KEY,
@@ -346,7 +282,6 @@ def test_duckdb_complex_queries(duckdb_session: DuckDBDriver) -> None:
             (5, 'Eve', 3, 60000.00);
     """)
 
-    # Test complex JOIN query with aggregation
     complex_query = """
         SELECT
             d.dept_name,
@@ -362,12 +297,10 @@ def test_duckdb_complex_queries(duckdb_session: DuckDBDriver) -> None:
     result = duckdb_session.execute(complex_query)
     assert result.total_count == 3
 
-    # Engineering should have highest average salary
     engineering_row = next(row for row in result.data if row["dept_name"] == "Engineering")
     assert engineering_row["employee_count"] == 2
     assert engineering_row["avg_salary"] == 77500.0
 
-    # Test subquery
     subquery = """
         SELECT emp_name, salary
         FROM employees
@@ -376,16 +309,14 @@ def test_duckdb_complex_queries(duckdb_session: DuckDBDriver) -> None:
     """
 
     subquery_result = duckdb_session.execute(subquery)
-    assert len(subquery_result.data) >= 1  # At least one employee above average
+    assert len(subquery_result.data) >= 1
 
-    # Clean up
     duckdb_session.execute_script("DROP TABLE employees; DROP TABLE departments;")
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_window_functions(duckdb_session: DuckDBDriver) -> None:
     """Test DuckDB window functions."""
-    # Create test data for window functions
+
     duckdb_session.execute_script("""
         CREATE TABLE sales_data (
             id INTEGER,
@@ -402,7 +333,6 @@ def test_duckdb_window_functions(duckdb_session: DuckDBDriver) -> None:
             (5, 'Product B', 1800.00, '2024-01-05');
     """)
 
-    # Test window function with ranking
     window_query = """
         SELECT
             product,
@@ -417,19 +347,16 @@ def test_duckdb_window_functions(duckdb_session: DuckDBDriver) -> None:
     result = duckdb_session.execute(window_query)
     assert result.total_count == 5
 
-    # Verify window function results
     product_a_rows = [row for row in result.data if row["product"] == "Product A"]
     assert len(product_a_rows) == 2
-    assert product_a_rows[0]["rank_in_product"] == 1  # Highest sales amount ranked 1
+    assert product_a_rows[0]["rank_in_product"] == 1
 
-    # Clean up
     duckdb_session.execute_script("DROP TABLE sales_data")
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_schema_operations(duckdb_session: DuckDBDriver) -> None:
     """Test DuckDB schema operations (DDL)."""
-    # Test CREATE TABLE
+
     create_result = duckdb_session.execute("""
         CREATE TABLE schema_test (
             id INTEGER PRIMARY KEY,
@@ -439,15 +366,12 @@ def test_duckdb_schema_operations(duckdb_session: DuckDBDriver) -> None:
     """)
     assert isinstance(create_result, SQLResult)
 
-    # Test ALTER TABLE
     alter_result = duckdb_session.execute("ALTER TABLE schema_test ADD COLUMN email TEXT")
     assert isinstance(alter_result, SQLResult)
 
-    # Test CREATE INDEX
     index_result = duckdb_session.execute("CREATE INDEX idx_schema_test_name ON schema_test(name)")
     assert isinstance(index_result, SQLResult)
 
-    # Verify table structure by inserting and querying
     insert_result = duckdb_session.execute(
         "INSERT INTO schema_test (id, name, email) VALUES (?, ?, ?)", [1, "Test User", "test@example.com"]
     )
@@ -458,15 +382,13 @@ def test_duckdb_schema_operations(duckdb_session: DuckDBDriver) -> None:
     assert select_result.data[0]["name"] == "Test User"
     assert select_result.data[0]["email"] == "test@example.com"
 
-    # Test DROP operations
     duckdb_session.execute("DROP INDEX idx_schema_test_name")
     duckdb_session.execute("DROP TABLE schema_test")
 
 
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_performance_bulk_operations(duckdb_session: DuckDBDriver) -> None:
     """Test DuckDB performance with bulk operations."""
-    # Create table for bulk testing
+
     duckdb_session.execute_script("""
         CREATE TABLE bulk_test (
             id INTEGER,
@@ -475,19 +397,15 @@ def test_duckdb_performance_bulk_operations(duckdb_session: DuckDBDriver) -> Non
         )
     """)
 
-    # Generate bulk data (100 records)
     bulk_data = [(i, f"value_{i}", float(i * 10.5)) for i in range(1, 101)]
 
-    # Test bulk insert
     bulk_insert_sql = "INSERT INTO bulk_test (id, value, number) VALUES (?, ?, ?)"
     bulk_result = duckdb_session.execute_many(bulk_insert_sql, bulk_data)
     assert bulk_result.rows_affected == 100
 
-    # Test bulk query performance
     bulk_select_result = duckdb_session.execute("SELECT COUNT(*) as total FROM bulk_test")
     assert bulk_select_result.data[0]["total"] == 100
 
-    # Test aggregation on bulk data
     agg_result = duckdb_session.execute("""
         SELECT
             COUNT(*) as count,
@@ -502,82 +420,15 @@ def test_duckdb_performance_bulk_operations(duckdb_session: DuckDBDriver) -> Non
     assert agg_result.data[0]["min_number"] == 10.5
     assert agg_result.data[0]["max_number"] == 1050.0
 
-    # Clean up
     duckdb_session.execute_script("DROP TABLE bulk_test")
 
 
-@pytest.mark.xdist_group("duckdb")
-def test_duckdb_arrow_integration_comprehensive(duckdb_session: DuckDBDriver) -> None:
-    """Test comprehensive Arrow integration with DuckDB."""
-    if not hasattr(duckdb_session, "fetch_arrow_table"):
-        pytest.skip("DuckDB driver does not support Arrow operations")
-
-    # Create table with various data types for Arrow testing
-    duckdb_session.execute_script("""
-        CREATE TABLE arrow_test (
-            id INTEGER,
-            name TEXT,
-            value DOUBLE,
-            active BOOLEAN,
-            created_date DATE
-        );
-
-        INSERT INTO arrow_test VALUES
-            (1, 'Alice', 123.45, true, '2024-01-01'),
-            (2, 'Bob', 234.56, false, '2024-01-02'),
-            (3, 'Carol', 345.67, true, '2024-01-03'),
-            (4, 'Dave', 456.78, false, '2024-01-04'),
-            (5, 'Eve', 567.89, true, '2024-01-05');
-    """)
-
-    # Test Arrow result with filtering
-    arrow_result = duckdb_session.fetch_arrow_table(
-        "SELECT id, name, value FROM arrow_test WHERE active = ? ORDER BY id", parameters=[True]
-    )
-
-    assert isinstance(arrow_result, ArrowResult)
-    arrow_table = arrow_result.data
-    assert isinstance(arrow_table, pa.Table)
-    assert arrow_table.num_rows == 3  # 3 active records
-    assert arrow_table.num_columns == 3
-    assert arrow_table.column_names == ["id", "name", "value"]
-
-    # Verify Arrow data
-    ids = arrow_table.column("id").to_pylist()
-    names = arrow_table.column("name").to_pylist()
-    values = arrow_table.column("value").to_pylist()
-
-    assert ids == [1, 3, 5]
-    assert names == ["Alice", "Carol", "Eve"]
-    assert values == [123.45, 345.67, 567.89]
-
-    # Test Arrow with aggregation
-    agg_arrow_result = duckdb_session.fetch_arrow_table("""
-        SELECT
-            active,
-            COUNT(*) as count,
-            AVG(value) as avg_value
-        FROM arrow_test
-        GROUP BY active
-        ORDER BY active
-    """)
-
-    agg_table = agg_arrow_result.data
-    assert agg_table.num_rows == 2  # true and false groups
-    assert agg_table.num_columns == 3
-
-    # Clean up
-    duckdb_session.execute_script("DROP TABLE arrow_test")
-
-
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_error_handling_and_edge_cases(duckdb_session: DuckDBDriver) -> None:
     """Test DuckDB error handling and edge cases."""
-    # Test invalid SQL
+
     with pytest.raises(Exception):
         duckdb_session.execute("INVALID SQL STATEMENT")
 
-    # Test constraint violation
     duckdb_session.execute_script("""
         CREATE TABLE constraint_test (
             id INTEGER PRIMARY KEY,
@@ -585,73 +436,21 @@ def test_duckdb_error_handling_and_edge_cases(duckdb_session: DuckDBDriver) -> N
         )
     """)
 
-    # Test NOT NULL constraint violation
     with pytest.raises(Exception):
         duckdb_session.execute("INSERT INTO constraint_test (id) VALUES (1)")
 
-    # Test valid insert after constraint error
     valid_result = duckdb_session.execute("INSERT INTO constraint_test (id, name) VALUES (?, ?)", [1, "Valid Name"])
     assert valid_result.rows_affected == 1
 
-    # Test duplicate primary key
     with pytest.raises(Exception):
         duckdb_session.execute("INSERT INTO constraint_test (id, name) VALUES (?, ?)", [1, "Duplicate ID"])
 
-    # Clean up
     duckdb_session.execute_script("DROP TABLE constraint_test")
 
 
-@pytest.mark.xdist_group("duckdb")
-def test_duckdb_with_schema_type_conversion(duckdb_session: DuckDBDriver) -> None:
-    """Test DuckDB driver with schema type conversion."""
-    from dataclasses import dataclass
-
-    @dataclass
-    class TestRecord:
-        id: int
-        name: str
-        value: float | None = None
-
-    # Create test data
-    duckdb_session.execute_script("""
-        CREATE TABLE schema_conversion_test (
-            id INTEGER,
-            name TEXT,
-            value DOUBLE
-        );
-
-        INSERT INTO schema_conversion_test VALUES
-            (1, 'Record 1', 100.5),
-            (2, 'Record 2', 200.75),
-            (3, 'Record 3', NULL);
-    """)
-
-    # Test schema type conversion
-    result = duckdb_session.execute(
-        "SELECT id, name, value FROM schema_conversion_test ORDER BY id", schema_type=TestRecord
-    )
-
-    assert isinstance(result, SQLResult)
-    assert result.total_count == 3
-
-    # Verify converted data types
-    for i, record in enumerate(result.data, 1):
-        assert isinstance(record, TestRecord)
-        assert record.id == i
-        assert record.name == f"Record {i}"
-        if i < 3:
-            assert record.value is not None
-        else:
-            assert record.value is None
-
-    # Clean up
-    duckdb_session.execute_script("DROP TABLE schema_conversion_test")
-
-
-@pytest.mark.xdist_group("duckdb")
 def test_duckdb_result_methods_comprehensive(duckdb_session: DuckDBDriver) -> None:
-    """Test comprehensive SelectResult and ExecuteResult methods."""
-    # Test SelectResult methods
+    """Test comprehensive SQLResult methods."""
+
     duckdb_session.execute_script("""
         CREATE TABLE result_methods_test (
             id INTEGER,
@@ -666,37 +465,29 @@ def test_duckdb_result_methods_comprehensive(duckdb_session: DuckDBDriver) -> No
             (4, 'C', 40);
     """)
 
-    # Test SelectResult methods
     select_result = duckdb_session.execute("SELECT * FROM result_methods_test ORDER BY id")
 
-    # Test get_count()
     assert select_result.get_count() == 4
 
-    # Test get_first()
     first_row = select_result.get_first()
     assert first_row is not None
     assert first_row["id"] == 1
 
-    # Test is_empty()
     assert not select_result.is_empty()
 
-    # Test empty result
     empty_result = duckdb_session.execute("SELECT * FROM result_methods_test WHERE id > 100")
     assert empty_result.is_empty()
     assert empty_result.get_count() == 0
     assert empty_result.get_first() is None
 
-    # Test ExecuteResult methods
     update_result = duckdb_session.execute("UPDATE result_methods_test SET value = value * 2 WHERE category = 'A'")
 
-    # Test ExecuteResult methods
     assert isinstance(update_result, SQLResult)
     assert update_result.get_affected_count() == 2
     assert update_result.was_updated()
     assert not update_result.was_inserted()
     assert not update_result.was_deleted()
 
-    # Test INSERT result
     insert_result = duckdb_session.execute(
         "INSERT INTO result_methods_test (id, category, value) VALUES (?, ?, ?)", [5, "D", 50]
     )
@@ -704,32 +495,116 @@ def test_duckdb_result_methods_comprehensive(duckdb_session: DuckDBDriver) -> No
     assert insert_result.was_inserted()
     assert insert_result.get_affected_count() == 1
 
-    # Test DELETE result
     delete_result = duckdb_session.execute("DELETE FROM result_methods_test WHERE category = 'C'")
     assert isinstance(delete_result, SQLResult)
     assert delete_result.was_deleted()
     assert delete_result.get_affected_count() == 1
 
-    # Clean up
     duckdb_session.execute_script("DROP TABLE result_methods_test")
 
 
-@pytest.mark.xdist_group("duckdb")
-def test_duckdb_to_parquet(duckdb_session: DuckDBDriver) -> None:
-    """Integration test: to_parquet writes correct data to a Parquet file using DuckDB native API."""
-    duckdb_session.execute("CREATE TABLE IF NOT EXISTS test_table (id INTEGER, name VARCHAR)")
-    duckdb_session.execute("INSERT INTO test_table (id, name) VALUES (?, ?)", (1, "arrow1"))
-    duckdb_session.execute("INSERT INTO test_table (id, name) VALUES (?, ?)", (2, "arrow2"))
-    statement = SQL("SELECT id, name FROM test_table ORDER BY id")
-    with tempfile.TemporaryDirectory() as tmpdir:
-        output_path = Path(tmpdir) / "partitioned_data"
-        try:
-            duckdb_session.export_to_storage(statement, destination_uri=str(output_path))  # type: ignore[attr-defined]
-            table = pq.read_table(f"{output_path}.parquet")
-            assert table.num_rows == 2
-            assert table.column_names == ["id", "name"]
-            data = table.to_pylist()
-            assert data[0]["id"] == 1 and data[0]["name"] == "arrow1"
-            assert data[1]["id"] == 2 and data[1]["name"] == "arrow2"
-        except Exception as e:
-            pytest.fail(f"Failed to export to storage: {e}")
+def test_duckdb_for_update_locking(duckdb_session: DuckDBDriver) -> None:
+    """Test FOR UPDATE row locking with DuckDB (may have limited support)."""
+    from sqlspec import sql
+
+    # Setup test table
+    duckdb_session.execute_script("DROP TABLE IF EXISTS test_table")
+    duckdb_session.execute_script("""
+        CREATE TABLE test_table (
+            id INTEGER PRIMARY KEY,
+            name VARCHAR,
+            value INTEGER
+        )
+    """)
+
+    # Insert test data
+    duckdb_session.execute("INSERT INTO test_table (id, name, value) VALUES (?, ?, ?)", (1, "duckdb_lock", 100))
+
+    try:
+        duckdb_session.begin()
+
+        # Test basic FOR UPDATE (DuckDB may have limited or no support)
+        result = duckdb_session.select_one(
+            sql.select("id", "name", "value").from_("test_table").where_eq("name", "duckdb_lock").for_update()
+        )
+        assert result is not None
+        assert result["name"] == "duckdb_lock"
+        assert result["value"] == 100
+
+        duckdb_session.commit()
+    except Exception:
+        duckdb_session.rollback()
+        raise
+    finally:
+        duckdb_session.execute_script("DROP TABLE IF EXISTS test_table")
+
+
+def test_duckdb_for_update_nowait(duckdb_session: DuckDBDriver) -> None:
+    """Test FOR UPDATE NOWAIT with DuckDB."""
+    from sqlspec import sql
+
+    # Setup test table
+    duckdb_session.execute_script("DROP TABLE IF EXISTS test_table")
+    duckdb_session.execute_script("""
+        CREATE TABLE test_table (
+            id INTEGER PRIMARY KEY,
+            name VARCHAR,
+            value INTEGER
+        )
+    """)
+
+    # Insert test data
+    duckdb_session.execute("INSERT INTO test_table (id, name, value) VALUES (?, ?, ?)", (1, "duckdb_nowait", 200))
+
+    try:
+        duckdb_session.begin()
+
+        # Test FOR UPDATE NOWAIT
+        result = duckdb_session.select_one(
+            sql.select("*").from_("test_table").where_eq("name", "duckdb_nowait").for_update(nowait=True)
+        )
+        assert result is not None
+        assert result["name"] == "duckdb_nowait"
+
+        duckdb_session.commit()
+    except Exception:
+        duckdb_session.rollback()
+        raise
+    finally:
+        duckdb_session.execute_script("DROP TABLE IF EXISTS test_table")
+
+
+def test_duckdb_for_share_locking(duckdb_session: DuckDBDriver) -> None:
+    """Test FOR SHARE row locking with DuckDB."""
+    from sqlspec import sql
+
+    # Setup test table
+    duckdb_session.execute_script("DROP TABLE IF EXISTS test_table")
+    duckdb_session.execute_script("""
+        CREATE TABLE test_table (
+            id INTEGER PRIMARY KEY,
+            name VARCHAR,
+            value INTEGER
+        )
+    """)
+
+    # Insert test data
+    duckdb_session.execute("INSERT INTO test_table (id, name, value) VALUES (?, ?, ?)", (1, "duckdb_share", 300))
+
+    try:
+        duckdb_session.begin()
+
+        # Test FOR SHARE (DuckDB support may vary)
+        result = duckdb_session.select_one(
+            sql.select("id", "name", "value").from_("test_table").where_eq("name", "duckdb_share").for_share()
+        )
+        assert result is not None
+        assert result["name"] == "duckdb_share"
+        assert result["value"] == 300
+
+        duckdb_session.commit()
+    except Exception:
+        duckdb_session.rollback()
+        raise
+    finally:
+        duckdb_session.execute_script("DROP TABLE IF EXISTS test_table")
