@@ -3,13 +3,16 @@ from pytest_databases.docker.postgres import PostgresService
 
 from sqlspec.adapters.psycopg import PsycopgAsyncConfig, PsycopgSyncConfig
 
+pytestmark = pytest.mark.xdist_group("postgres")
+
 
-@pytest.mark.xdist_group("postgres")
 async def test_async_connection(postgres_service: PostgresService) -> None:
     """Test async connection components."""
     # Test direct connection
     async_config = PsycopgAsyncConfig(
-        conninfo=f"host={postgres_service.host} port={postgres_service.port} user={postgres_service.user} password={postgres_service.password} dbname={postgres_service.database}"
+        pool_config={
+            "conninfo": f"host={postgres_service.host} port={postgres_service.port} user={postgres_service.user} password={postgres_service.password} dbname={postgres_service.database}"
+        }
     )
 
     async with await async_config.create_connection() as conn:
@@ -22,14 +25,14 @@ async def test_async_connection(postgres_service: PostgresService) -> None:
             assert result == {"id": 1}
 
     # Ensure pool is closed properly with timeout
-    if async_config.pool_instance:
-        await async_config.pool_instance.close(timeout=5.0)
-        async_config.pool_instance = None
+    await async_config.close_pool()
     # Test connection pool
     another_config = PsycopgAsyncConfig(
-        conninfo=f"host={postgres_service.host} port={postgres_service.port} user={postgres_service.user} password={postgres_service.password} dbname={postgres_service.database}",
-        min_size=1,
-        max_size=5,
+        pool_config={
+            "conninfo": f"host={postgres_service.host} port={postgres_service.port} user={postgres_service.user} password={postgres_service.password} dbname={postgres_service.database}",
+            "min_size": 1,
+            "max_size": 5,
+        }
     )
     # Remove explicit pool creation and manual context management
     async with another_config.provide_connection() as conn:
@@ -41,17 +44,16 @@ async def test_async_connection(postgres_service: PostgresService) -> None:
             assert result == {"value": 1}  # type: ignore[comparison-overlap]
 
     # Ensure pool is closed properly with timeout
-    if another_config.pool_instance:
-        await another_config.pool_instance.close(timeout=5.0)
-        another_config.pool_instance = None
+    await another_config.close_pool()
 
 
-@pytest.mark.xdist_group("postgres")
 def test_sync_connection(postgres_service: PostgresService) -> None:
     """Test sync connection components."""
     # Test direct connection
     sync_config = PsycopgSyncConfig(
-        conninfo=f"host={postgres_service.host} port={postgres_service.port} user={postgres_service.user} password={postgres_service.password} dbname={postgres_service.database}"
+        pool_config={
+            "conninfo": f"host={postgres_service.host} port={postgres_service.port} user={postgres_service.user} password={postgres_service.password} dbname={postgres_service.database}"
+        }
     )
 
     try:
@@ -64,15 +66,15 @@ def test_sync_connection(postgres_service: PostgresService) -> None:
                 assert result == {"id": 1}
     finally:
         # Ensure pool is closed properly with timeout
-        if sync_config.pool_instance:
-            sync_config.pool_instance.close(timeout=5.0)
-            sync_config.pool_instance = None
+        sync_config.close_pool()
 
     # Test connection pool
     another_config = PsycopgSyncConfig(
-        conninfo=f"postgres://{postgres_service.user}:{postgres_service.password}@{postgres_service.host}:{postgres_service.port}/{postgres_service.database}",
-        min_size=1,
-        max_size=5,
+        pool_config={
+            "conninfo": f"postgres://{postgres_service.user}:{postgres_service.password}@{postgres_service.host}:{postgres_service.port}/{postgres_service.database}",
+            "min_size": 1,
+            "max_size": 5,
+        }
     )
     try:
         # Remove explicit pool creation and manual context management
@@ -85,6 +87,4 @@ def test_sync_connection(postgres_service: PostgresService) -> None:
                 assert result == {"id": 1}
     finally:
         # Ensure pool is closed properly with timeout
-        if another_config.pool_instance:
-            another_config.pool_instance.close(timeout=5.0)
-            another_config.pool_instance = None
+        another_config.close_pool()
