@@ -2,39 +2,31 @@
 
 from __future__ import annotations
 
-import tempfile
 from collections.abc import Generator
 from typing import Any, Literal
 
-import pyarrow.parquet as pq
 import pytest
 from pytest_databases.docker.postgres import PostgresService
 
 from sqlspec.adapters.psycopg import PsycopgSyncConfig, PsycopgSyncDriver
-from sqlspec.statement.result import ArrowResult, SQLResult
-from sqlspec.statement.sql import SQL
+from sqlspec.core.result import SQLResult
 
 ParamStyle = Literal["tuple_binds", "dict_binds", "named_binds"]
 
+pytestmark = pytest.mark.xdist_group("postgres")
+
 
 @pytest.fixture
 def psycopg_session(postgres_service: PostgresService) -> Generator[PsycopgSyncDriver, None, None]:
     """Create a psycopg session with test table."""
-    from sqlspec.statement.sql import SQLConfig
-
     config = PsycopgSyncConfig(
-        host=postgres_service.host,
-        port=postgres_service.port,
-        user=postgres_service.user,
-        password=postgres_service.password,
-        dbname=postgres_service.database,
-        autocommit=True,  # Enable autocommit for tests
-        statement_config=SQLConfig(enable_transformations=False, enable_validation=False, enable_parsing=False),
+        pool_config={
+            "conninfo": f"postgresql://{postgres_service.user}:{postgres_service.password}@{postgres_service.host}:{postgres_service.port}/{postgres_service.database}"
+        }
     )
 
     try:
         with config.provide_session() as session:
-            # Create test table
             session.execute_script("""
                 CREATE TABLE IF NOT EXISTS test_table (
                     id SERIAL PRIMARY KEY,
@@ -43,62 +35,61 @@ def psycopg_session(postgres_service: PostgresService) -> Generator[PsycopgSyncD
                     created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
                 )
             """)
+
+            session.commit()
+
+            session.begin()
             yield session
-            # Cleanup - handle potential transaction errors
+
+            try:
+                session.rollback()
+            except Exception:
+                pass
+
             try:
                 session.execute_script("DROP TABLE IF EXISTS test_table")
             except Exception:
-                # If the transaction is in an error state, rollback first
                 if hasattr(session.connection, "rollback"):
-                    session.connection.rollback()
-                # Try again after rollback
+                    try:
+                        session.connection.rollback()
+                    except Exception:
+                        pass
+
                 try:
                     session.execute_script("DROP TABLE IF EXISTS test_table")
                 except Exception:
-                    # If it still fails, ignore - table might not exist
                     pass
     finally:
-        # Ensure pool is closed properly to avoid "cannot join current thread" warnings
-        if config.pool_instance:
-            config.pool_instance.close(timeout=5.0)
-            config.pool_instance = None
+        config.close_pool()
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_basic_crud(psycopg_session: PsycopgSyncDriver) -> None:
     """Test basic CRUD operations."""
-    # INSERT
-    insert_result = psycopg_session.execute(
-        "INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("test_name", 42)
-    )
+
+    insert_result = psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", "test_name", 42)
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # SELECT
-    select_result = psycopg_session.execute("SELECT name, value FROM test_table WHERE name = %s", ("test_name"))
+    select_result = psycopg_session.execute("SELECT name, value FROM test_table WHERE name = %s", "test_name")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
     assert len(select_result.data) == 1
     assert select_result.data[0]["name"] == "test_name"
     assert select_result.data[0]["value"] == 42
 
-    # UPDATE
-    update_result = psycopg_session.execute("UPDATE test_table SET value = %s WHERE name = %s", (100, "test_name"))
+    update_result = psycopg_session.execute("UPDATE test_table SET value = %s WHERE name = %s", 100, "test_name")
     assert isinstance(update_result, SQLResult)
     assert update_result.rows_affected == 1
 
-    # Verify UPDATE
-    verify_result = psycopg_session.execute("SELECT value FROM test_table WHERE name = %s", ("test_name"))
+    verify_result = psycopg_session.execute("SELECT value FROM test_table WHERE name = %s", "test_name")
     assert isinstance(verify_result, SQLResult)
     assert verify_result.data is not None
     assert verify_result.data[0]["value"] == 100
 
-    # DELETE
-    delete_result = psycopg_session.execute("DELETE FROM test_table WHERE name = %s", ("test_name"))
+    delete_result = psycopg_session.execute("DELETE FROM test_table WHERE name = %s", "test_name")
     assert isinstance(delete_result, SQLResult)
     assert delete_result.rows_affected == 1
 
-    # Verify DELETE
     empty_result = psycopg_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(empty_result, SQLResult)
     assert empty_result.data is not None
@@ -106,49 +97,45 @@ def test_psycopg_basic_crud(psycopg_session: PsycopgSyncDriver) -> None:
 
 
 @pytest.mark.parametrize(
-    ("params", "style"),
+    ("parameters", "style"),
     [
         pytest.param(("test_value",), "tuple_binds", id="tuple_binds"),
         pytest.param({"name": "test_value"}, "dict_binds", id="dict_binds"),
     ],
 )
-@pytest.mark.xdist_group("postgres")
-def test_psycopg_parameter_styles(psycopg_session: PsycopgSyncDriver, params: Any, style: ParamStyle) -> None:
+def test_psycopg_parameter_styles(psycopg_session: PsycopgSyncDriver, parameters: Any, style: ParamStyle) -> None:
     """Test different parameter binding styles."""
-    # Insert test data
+
     psycopg_session.execute("INSERT INTO test_table (name) VALUES (%s)", "test_value")
 
-    # Test parameter style
     if style == "tuple_binds":
         sql = "SELECT name FROM test_table WHERE name = %s"
-    else:  # dict_binds
+
+        result = psycopg_session.execute(sql, *parameters)
+    else:
         sql = "SELECT name FROM test_table WHERE name = %(name)s"
 
-    result = psycopg_session.execute(sql, params)
+        result = psycopg_session.execute(sql, **parameters)
+
     assert isinstance(result, SQLResult)
     assert result.data is not None
     assert len(result) == 1
     assert result.data[0]["name"] == "test_value"
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_execute_many(psycopg_session: PsycopgSyncDriver) -> None:
     """Test execute_many functionality."""
-    params_list = [("name1", 1), ("name2", 2), ("name3", 3)]
+    parameters_list = [("name1", 1), ("name2", 2), ("name3", 3)]
 
-    result = psycopg_session.execute_many(
-        "INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=params_list
-    )
+    result = psycopg_session.execute_many("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters_list)
     assert isinstance(result, SQLResult)
-    assert result.rows_affected == len(params_list)
+    assert result.rows_affected == len(parameters_list)
 
-    # Verify all records were inserted
     select_result = psycopg_session.execute("SELECT COUNT(*) as count FROM test_table")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
-    assert select_result.data[0]["count"] == len(params_list)
+    assert select_result.data[0]["count"] == len(parameters_list)
 
-    # Verify data integrity
     ordered_result = psycopg_session.execute("SELECT name, value FROM test_table ORDER BY name")
     assert isinstance(ordered_result, SQLResult)
     assert ordered_result.data is not None
@@ -157,7 +144,6 @@ def test_psycopg_execute_many(psycopg_session: PsycopgSyncDriver) -> None:
     assert ordered_result.data[0]["value"] == 1
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_execute_script(psycopg_session: PsycopgSyncDriver) -> None:
     """Test execute_script functionality."""
     script = """
@@ -167,11 +153,10 @@ def test_psycopg_execute_script(psycopg_session: PsycopgSyncDriver) -> None:
     """
 
     result = psycopg_session.execute_script(script)
-    # Script execution returns a SQLResult
+
     assert isinstance(result, SQLResult)
     assert result.operation_type == "SCRIPT"
 
-    # Verify script effects
     select_result = psycopg_session.execute(
         "SELECT name, value FROM test_table WHERE name LIKE 'script_test%' ORDER BY name"
     )
@@ -184,56 +169,48 @@ def test_psycopg_execute_script(psycopg_session: PsycopgSyncDriver) -> None:
     assert select_result.data[1]["value"] == 888
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_result_methods(psycopg_session: PsycopgSyncDriver) -> None:
     """Test SelectResult and ExecuteResult methods."""
-    # Insert test data
+
     psycopg_session.execute_many(
-        "INSERT INTO test_table (name, value) VALUES (%s, %s)",
-        parameters=[("result1", 10), ("result2", 20), ("result3", 30)],
+        "INSERT INTO test_table (name, value) VALUES (%s, %s)", [("result1", 10), ("result2", 20), ("result3", 30)]
     )
 
-    # Test SelectResult methods
     result = psycopg_session.execute("SELECT * FROM test_table ORDER BY name")
     assert isinstance(result, SQLResult)
 
-    # Test get_first()
     first_row = result.get_first()
     assert first_row is not None
     assert first_row["name"] == "result1"
 
-    # Test get_count()
     assert result.get_count() == 3
 
-    # Test is_empty()
     assert not result.is_empty()
 
-    # Test empty result
-    empty_result = psycopg_session.execute("SELECT * FROM test_table WHERE name = %s", ("nonexistent"))
+    empty_result = psycopg_session.execute("SELECT * FROM test_table WHERE name = %s", "nonexistent")
     assert isinstance(empty_result, SQLResult)
     assert empty_result.is_empty()
     assert empty_result.get_first() is None
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_error_handling(psycopg_session: PsycopgSyncDriver) -> None:
     """Test error handling and exception propagation."""
-    # Test invalid SQL
-    with pytest.raises(Exception):  # psycopg.errors.SyntaxError
+
+    with pytest.raises(Exception):
         psycopg_session.execute("INVALID SQL STATEMENT")
 
-    # Test constraint violation
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("unique_test", 1))
+    psycopg_session.rollback()
+    psycopg_session.begin()
 
-    # Try to insert with invalid column reference
-    with pytest.raises(Exception):  # psycopg.errors.UndefinedColumn
+    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", "unique_test", 1)
+
+    with pytest.raises(Exception):
         psycopg_session.execute("SELECT nonexistent_column FROM test_table")
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_data_types(psycopg_session: PsycopgSyncDriver) -> None:
     """Test PostgreSQL data type handling with psycopg."""
-    # Create table with various PostgreSQL data types
+
     psycopg_session.execute_script("""
         CREATE TABLE data_types_test (
             id SERIAL PRIMARY KEY,
@@ -249,7 +226,6 @@ def test_psycopg_data_types(psycopg_session: PsycopgSyncDriver) -> None:
         )
     """)
 
-    # Insert data with various types
     psycopg_session.execute(
         """
         INSERT INTO data_types_test (
@@ -259,20 +235,17 @@ def test_psycopg_data_types(psycopg_session: PsycopgSyncDriver) -> None:
             %s, %s, %s, %s, %s, %s, %s, %s, %s
         )
     """,
-        parameters=(
-            "text_value",
-            42,
-            123.45,
-            True,
-            '{"key": "value"}',
-            [1, 2, 3],
-            "2024-01-15",
-            "2024-01-15 10:30:00",
-            "550e8400-e29b-41d4-a716-446655440000",
-        ),
+        "text_value",
+        42,
+        123.45,
+        True,
+        '{"key": "value"}',
+        [1, 2, 3],
+        "2024-01-15",
+        "2024-01-15 10:30:00",
+        "550e8400-e29b-41d4-a716-446655440000",
     )
 
-    # Retrieve and verify data
     select_result = psycopg_session.execute(
         "SELECT text_col, integer_col, numeric_col, boolean_col, json_col, array_col FROM data_types_test"
     )
@@ -286,34 +259,27 @@ def test_psycopg_data_types(psycopg_session: PsycopgSyncDriver) -> None:
     assert row["boolean_col"] is True
     assert row["array_col"] == [1, 2, 3]
 
-    # Clean up
     psycopg_session.execute_script("DROP TABLE data_types_test")
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_transactions(psycopg_session: PsycopgSyncDriver) -> None:
     """Test transaction behavior."""
-    # PostgreSQL supports explicit transactions
-    psycopg_session.execute(
-        "INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("transaction_test", 100)
-    )
 
-    # Verify data is committed
+    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", "transaction_test", 100)
+
     result = psycopg_session.execute("SELECT COUNT(*) as count FROM test_table WHERE name = %s", ("transaction_test"))
     assert isinstance(result, SQLResult)
     assert result.data is not None
     assert result.data[0]["count"] == 1
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_complex_queries(psycopg_session: PsycopgSyncDriver) -> None:
     """Test complex SQL queries."""
-    # Insert test data
+
     test_data = [("Alice", 25), ("Bob", 30), ("Charlie", 35), ("Diana", 28)]
 
-    psycopg_session.execute_many("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=test_data)
+    psycopg_session.execute_many("INSERT INTO test_table (name, value) VALUES (%s, %s)", test_data)
 
-    # Test JOIN (self-join)
     join_result = psycopg_session.execute("""
         SELECT t1.name as name1, t2.name as name2, t1.value as value1, t2.value as value2
         FROM test_table t1
@@ -326,7 +292,6 @@ def test_psycopg_complex_queries(psycopg_session: PsycopgSyncDriver) -> None:
     assert join_result.data is not None
     assert len(join_result.data) == 3
 
-    # Test aggregation
     agg_result = psycopg_session.execute("""
         SELECT
             COUNT(*) as total_count,
@@ -342,7 +307,6 @@ def test_psycopg_complex_queries(psycopg_session: PsycopgSyncDriver) -> None:
     assert agg_result.data[0]["min_value"] == 25
     assert agg_result.data[0]["max_value"] == 35
 
-    # Test subquery
     subquery_result = psycopg_session.execute("""
         SELECT name, value
         FROM test_table
@@ -351,15 +315,14 @@ def test_psycopg_complex_queries(psycopg_session: PsycopgSyncDriver) -> None:
     """)
     assert isinstance(subquery_result, SQLResult)
     assert subquery_result.data is not None
-    assert len(subquery_result.data) == 2  # Bob and Charlie
+    assert len(subquery_result.data) == 2
     assert subquery_result.data[0]["name"] == "Bob"
     assert subquery_result.data[1]["name"] == "Charlie"
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_schema_operations(psycopg_session: PsycopgSyncDriver) -> None:
     """Test schema operations (DDL)."""
-    # Create a new table
+
     psycopg_session.execute_script("""
         CREATE TABLE schema_test (
             id SERIAL PRIMARY KEY,
@@ -368,14 +331,10 @@ def test_psycopg_schema_operations(psycopg_session: PsycopgSyncDriver) -> None:
         )
     """)
 
-    # Insert data into new table
-    insert_result = psycopg_session.execute(
-        "INSERT INTO schema_test (description) VALUES (%s)", parameters=("test description",)
-    )
+    insert_result = psycopg_session.execute("INSERT INTO schema_test (description) VALUES (%s)", "test description")
     assert isinstance(insert_result, SQLResult)
     assert insert_result.rows_affected == 1
 
-    # Verify table structure
     info_result = psycopg_session.execute("""
         SELECT column_name, data_type
         FROM information_schema.columns
@@ -384,19 +343,16 @@ def test_psycopg_schema_operations(psycopg_session: PsycopgSyncDriver) -> None:
     """)
     assert isinstance(info_result, SQLResult)
     assert info_result.data is not None
-    assert len(info_result.data) == 3  # id, description, created_at
+    assert len(info_result.data) == 3
 
-    # Drop table
     psycopg_session.execute_script("DROP TABLE schema_test")
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_column_names_and_metadata(psycopg_session: PsycopgSyncDriver) -> None:
     """Test column names and result metadata."""
-    # Insert test data
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("metadata_test", 123))
 
-    # Test column names
+    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", "metadata_test", 123)
+
     result = psycopg_session.execute(
         "SELECT id, name, value, created_at FROM test_table WHERE name = %s", ("metadata_test",)
     )
@@ -405,7 +361,6 @@ def test_psycopg_column_names_and_metadata(psycopg_session: PsycopgSyncDriver) -
     assert result.data is not None
     assert len(result) == 1
 
-    # Test that we can access data by column name
     row = result.data[0]
     assert row["name"] == "metadata_test"
     assert row["value"] == 123
@@ -413,51 +368,20 @@ def test_psycopg_column_names_and_metadata(psycopg_session: PsycopgSyncDriver) -
     assert row["created_at"] is not None
 
 
-@pytest.mark.xdist_group("postgres")
-def test_psycopg_with_schema_type(psycopg_session: PsycopgSyncDriver) -> None:
-    """Test psycopg driver with schema type conversion."""
-    from dataclasses import dataclass
-
-    @dataclass
-    class TestRecord:
-        id: int | None
-        name: str
-        value: int
-
-    # Insert test data
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("schema_test", 456))
-
-    # Query with schema type
-    result = psycopg_session.execute(
-        "SELECT id, name, value FROM test_table WHERE name = %s", "schema_test", schema_type=TestRecord
-    )
-
-    assert isinstance(result, SQLResult)
-    assert result.data is not None
-    assert len(result) == 1
-
-    # The data should be converted to the schema type by the ResultConverter
-    assert result.column_names == ["id", "name", "value"]
-
-
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_performance_bulk_operations(psycopg_session: PsycopgSyncDriver) -> None:
     """Test performance with bulk operations."""
-    # Generate bulk data
+
     bulk_data = [(f"bulk_user_{i}", i * 10) for i in range(100)]
 
-    # Bulk insert
-    result = psycopg_session.execute_many("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=bulk_data)
+    result = psycopg_session.execute_many("INSERT INTO test_table (name, value) VALUES (%s, %s)", bulk_data)
     assert isinstance(result, SQLResult)
     assert result.rows_affected == 100
 
-    # Bulk select
     select_result = psycopg_session.execute("SELECT COUNT(*) as count FROM test_table WHERE name LIKE 'bulk_user_%'")
     assert isinstance(select_result, SQLResult)
     assert select_result.data is not None
     assert select_result.data[0]["count"] == 100
 
-    # Test pagination-like query
     page_result = psycopg_session.execute(
         "SELECT name, value FROM test_table WHERE name LIKE 'bulk_user_%' ORDER BY value LIMIT 10 OFFSET 20"
     )
@@ -467,22 +391,19 @@ def test_psycopg_performance_bulk_operations(psycopg_session: PsycopgSyncDriver)
     assert page_result.data[0]["name"] == "bulk_user_20"
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_postgresql_specific_features(psycopg_session: PsycopgSyncDriver) -> None:
     """Test PostgreSQL-specific features with psycopg."""
-    # Test RETURNING clause
+
     returning_result = psycopg_session.execute(
-        "INSERT INTO test_table (name, value) VALUES (%s, %s) RETURNING id, name", parameters=("returning_test", 999)
+        "INSERT INTO test_table (name, value) VALUES (%s, %s) RETURNING id, name", "returning_test", 999
     )
-    assert isinstance(returning_result, SQLResult)  # psycopg returns SQLResult for RETURNING
+    assert isinstance(returning_result, SQLResult)
     assert returning_result.data is not None
     assert len(returning_result.data) == 1
     assert returning_result.data[0]["name"] == "returning_test"
 
-    # Test window functions
     psycopg_session.execute_many(
-        "INSERT INTO test_table (name, value) VALUES (%s, %s)",
-        parameters=[("window1", 10), ("window2", 20), ("window3", 30)],
+        "INSERT INTO test_table (name, value) VALUES (%s, %s)", [("window1", 10), ("window2", 20), ("window3", 30)]
     )
 
     window_result = psycopg_session.execute("""
@@ -502,36 +423,32 @@ def test_psycopg_postgresql_specific_features(psycopg_session: PsycopgSyncDriver
     assert window_result.data[0]["prev_value"] is None
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_json_operations(psycopg_session: PsycopgSyncDriver) -> None:
     """Test PostgreSQL JSON operations with psycopg."""
-    # Create table with JSONB column
+
     psycopg_session.execute_script("""
-        CREATE TABLE json_test (
+        CREATE TABLE IF NOT EXISTS json_test (
             id SERIAL PRIMARY KEY,
             data JSONB
-        )
+        );
+        DELETE FROM json_test;
     """)
 
-    # Insert JSON data
-    json_data = '{"name": "test", "age": 30, "tags": ["postgres", "json"]}'
-    psycopg_session.execute("INSERT INTO json_test (data) VALUES (%s)", parameters=(json_data,))
+    json_data = {"name": "test", "age": 30, "tags": ["postgres", "json"]}
+    psycopg_session.execute("INSERT INTO json_test (data) VALUES (%s)", (json_data,))
 
-    # Test JSON queries
     json_result = psycopg_session.execute("SELECT data->>'name' as name, data->>'age' as age FROM json_test")
     assert isinstance(json_result, SQLResult)
     assert json_result.data is not None
     assert json_result.data[0]["name"] == "test"
     assert json_result.data[0]["age"] == "30"
 
-    # Clean up
     psycopg_session.execute_script("DROP TABLE json_test")
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_copy_operations_positional(psycopg_session: PsycopgSyncDriver) -> None:
     """Test PostgreSQL COPY operations with psycopg using positional parameters."""
-    # Create temp table for copy test
+
     psycopg_session.execute_script("""
         DROP TABLE IF EXISTS copy_test_pos;
         CREATE TABLE copy_test_pos (
@@ -541,13 +458,11 @@ def test_psycopg_copy_operations_positional(psycopg_session: PsycopgSyncDriver)
         )
     """)
 
-    # Test COPY FROM STDIN with text format using positional parameter
     copy_data = "1\ttest1\t100\n2\ttest2\t200\n"
     result = psycopg_session.execute("COPY copy_test_pos FROM STDIN WITH (FORMAT text)", copy_data)
     assert isinstance(result, SQLResult)
-    assert result.rows_affected >= 0  # May be -1 or actual count
+    assert result.rows_affected >= 0
 
-    # Verify data was copied
     verify_result = psycopg_session.execute("SELECT * FROM copy_test_pos ORDER BY id")
     assert isinstance(verify_result, SQLResult)
     assert verify_result.data is not None
@@ -555,14 +470,12 @@ def test_psycopg_copy_operations_positional(psycopg_session: PsycopgSyncDriver)
     assert verify_result.data[0]["name"] == "test1"
     assert verify_result.data[1]["value"] == 200
 
-    # Clean up
     psycopg_session.execute_script("DROP TABLE copy_test_pos")
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_copy_operations_keyword(psycopg_session: PsycopgSyncDriver) -> None:
     """Test PostgreSQL COPY operations with psycopg using keyword parameters."""
-    # Create temp table for copy test
+
     psycopg_session.execute_script("""
         DROP TABLE IF EXISTS copy_test_kw;
         CREATE TABLE copy_test_kw (
@@ -572,13 +485,11 @@ def test_psycopg_copy_operations_keyword(psycopg_session: PsycopgSyncDriver) ->
         )
     """)
 
-    # Test COPY FROM STDIN with text format using keyword parameter
     copy_data = "3\ttest3\t300\n4\ttest4\t400\n"
-    result = psycopg_session.execute("COPY copy_test_kw FROM STDIN WITH (FORMAT text)", parameters=copy_data)
+    result = psycopg_session.execute("COPY copy_test_kw FROM STDIN WITH (FORMAT text)", copy_data)
     assert isinstance(result, SQLResult)
-    assert result.rows_affected >= 0  # May be -1 or actual count
+    assert result.rows_affected >= 0
 
-    # Verify data was copied
     verify_result = psycopg_session.execute("SELECT * FROM copy_test_kw ORDER BY id")
     assert isinstance(verify_result, SQLResult)
     assert verify_result.data is not None
@@ -586,14 +497,12 @@ def test_psycopg_copy_operations_keyword(psycopg_session: PsycopgSyncDriver) ->
     assert verify_result.data[0]["name"] == "test3"
     assert verify_result.data[1]["value"] == 400
 
-    # Clean up
     psycopg_session.execute_script("DROP TABLE copy_test_kw")
 
 
-@pytest.mark.xdist_group("postgres")
 def test_psycopg_copy_csv_format(psycopg_session: PsycopgSyncDriver) -> None:
     """Test PostgreSQL COPY operations with CSV format."""
-    # Create temp table
+
     psycopg_session.execute_script("""
         DROP TABLE IF EXISTS copy_csv_sync;
         CREATE TABLE copy_csv_sync (
@@ -603,75 +512,144 @@ def test_psycopg_copy_csv_format(psycopg_session: PsycopgSyncDriver) -> None:
         )
     """)
 
-    # Test COPY FROM STDIN with CSV format - positional
     csv_data = "5,test5,500\n6,test6,600\n7,test7,700\n"
     result_pos = psycopg_session.execute("COPY copy_csv_sync FROM STDIN WITH (FORMAT csv)", csv_data)
     assert isinstance(result_pos, SQLResult)
     assert result_pos.rows_affected == 3
 
-    # Verify data
     verify_result = psycopg_session.execute("SELECT * FROM copy_csv_sync ORDER BY id")
     assert isinstance(verify_result, SQLResult)
     assert len(verify_result.data) == 3
     assert verify_result.data[0]["name"] == "test5"
     assert verify_result.data[2]["value"] == 700
 
-    # Clear table and test with keyword parameter
     psycopg_session.execute_script("TRUNCATE TABLE copy_csv_sync")
 
     csv_data2 = "8,test8,800\n9,test9,900\n"
-    result_kw = psycopg_session.execute("COPY copy_csv_sync FROM STDIN WITH (FORMAT csv)", parameters=csv_data2)
+    result_kw = psycopg_session.execute("COPY copy_csv_sync FROM STDIN WITH (FORMAT csv)", csv_data2)
     assert isinstance(result_kw, SQLResult)
     assert result_kw.rows_affected == 2
 
-    # Verify data again
     verify_result2 = psycopg_session.execute("SELECT * FROM copy_csv_sync ORDER BY id")
     assert isinstance(verify_result2, SQLResult)
     assert len(verify_result2.data) == 2
     assert verify_result2.data[0]["name"] == "test8"
     assert verify_result2.data[1]["value"] == 900
 
-    # Clean up
     psycopg_session.execute_script("DROP TABLE copy_csv_sync")
 
 
-@pytest.mark.xdist_group("postgres")
-def test_psycopg_fetch_arrow_table(psycopg_session: PsycopgSyncDriver) -> None:
-    """Integration test: fetch_arrow_table returns ArrowResult with correct pyarrow.Table."""
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("arrow1", 111))
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("arrow2", 222))
-    statement = SQL("SELECT name, value FROM test_table ORDER BY name")
-    result = psycopg_session.fetch_arrow_table(statement)
-    assert isinstance(result, ArrowResult)
-    assert result.num_rows == 2
-    assert set(result.column_names) == {"name", "value"}
-    names = result.data["name"].to_pylist()
-    assert "arrow1" in names and "arrow2" in names
+@pytest.mark.integration
+def test_psycopg_sync_pgvector_integration(psycopg_session: PsycopgSyncDriver) -> None:
+    """Test that psycopg sync driver initializes pgvector support automatically via pool configure."""
+
+    result = psycopg_session.execute("SELECT 1 as test_value")
+    assert result.data is not None
+    assert result.data[0]["test_value"] == 1
 
 
-@pytest.mark.xdist_group("postgres")
-def test_psycopg_to_parquet(psycopg_session: PsycopgSyncDriver) -> None:
-    """Integration test: to_parquet writes correct data to a Parquet file."""
-    # Insert fresh data for this test
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("pq1", 123))
-    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", parameters=("pq2", 456))
+def test_psycopg_sync_for_update_locking(psycopg_session: PsycopgSyncDriver) -> None:
+    """Test FOR UPDATE row locking with psycopg (sync)."""
+    from sqlspec import sql
 
-    # First verify data can be selected normally
-    normal_result = psycopg_session.execute("SELECT name, value FROM test_table ORDER BY name")
-    assert len(normal_result.data) >= 2, f"Expected at least 2 rows, got {len(normal_result.data)}"
+    # Setup test table
+    psycopg_session.execute_script("DROP TABLE IF EXISTS test_table")
+    psycopg_session.execute_script("""
+        CREATE TABLE test_table (
+            id SERIAL PRIMARY KEY,
+            name VARCHAR(50),
+            value INTEGER
+        )
+    """)
 
-    # Use a simpler query without WHERE clause first
-    statement = "SELECT name, value FROM test_table ORDER BY name"
-    with tempfile.NamedTemporaryFile(suffix=".parquet", delete=False) as tmp:
-        try:
-            rows_exported = psycopg_session.export_to_storage(statement, destination_uri=tmp.name, format="parquet")
-            assert rows_exported == 2
-            table = pq.read_table(tmp.name)
-            assert table.num_rows == 2
-            assert set(table.column_names) == {"name", "value"}
-            names = table.column("name").to_pylist()
-            assert "pq1" in names and "pq2" in names
-        finally:
-            import os
+    # Insert test data
+    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", ("psycopg_lock", 100))
 
-            os.unlink(tmp.name)
+    try:
+        psycopg_session.begin()
+
+        # Test basic FOR UPDATE
+        result = psycopg_session.select_one(
+            sql.select("id", "name", "value").from_("test_table").where_eq("name", "psycopg_lock").for_update()
+        )
+        assert result is not None
+        assert result["name"] == "psycopg_lock"
+        assert result["value"] == 100
+
+        psycopg_session.commit()
+    except Exception:
+        psycopg_session.rollback()
+        raise
+    finally:
+        psycopg_session.execute_script("DROP TABLE IF EXISTS test_table")
+
+
+def test_psycopg_sync_for_update_skip_locked(psycopg_session: PsycopgSyncDriver) -> None:
+    """Test FOR UPDATE SKIP LOCKED with psycopg (sync)."""
+    from sqlspec import sql
+
+    # Setup test table
+    psycopg_session.execute_script("DROP TABLE IF EXISTS test_table")
+    psycopg_session.execute_script("""
+        CREATE TABLE test_table (
+            id SERIAL PRIMARY KEY,
+            name VARCHAR(50),
+            value INTEGER
+        )
+    """)
+
+    # Insert test data
+    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", ("psycopg_skip", 200))
+
+    try:
+        psycopg_session.begin()
+
+        # Test FOR UPDATE SKIP LOCKED
+        result = psycopg_session.select_one(
+            sql.select("*").from_("test_table").where_eq("name", "psycopg_skip").for_update(skip_locked=True)
+        )
+        assert result is not None
+        assert result["name"] == "psycopg_skip"
+
+        psycopg_session.commit()
+    except Exception:
+        psycopg_session.rollback()
+        raise
+    finally:
+        psycopg_session.execute_script("DROP TABLE IF EXISTS test_table")
+
+
+def test_psycopg_sync_for_share_locking(psycopg_session: PsycopgSyncDriver) -> None:
+    """Test FOR SHARE row locking with psycopg (sync)."""
+    from sqlspec import sql
+
+    # Setup test table
+    psycopg_session.execute_script("DROP TABLE IF EXISTS test_table")
+    psycopg_session.execute_script("""
+        CREATE TABLE test_table (
+            id SERIAL PRIMARY KEY,
+            name VARCHAR(50),
+            value INTEGER
+        )
+    """)
+
+    # Insert test data
+    psycopg_session.execute("INSERT INTO test_table (name, value) VALUES (%s, %s)", ("psycopg_share", 300))
+
+    try:
+        psycopg_session.begin()
+
+        # Test FOR SHARE
+        result = psycopg_session.select_one(
+            sql.select("id", "name", "value").from_("test_table").where_eq("name", "psycopg_share").for_share()
+        )
+        assert result is not None
+        assert result["name"] == "psycopg_share"
+        assert result["value"] == 300
+
+        psycopg_session.commit()
+    except Exception:
+        psycopg_session.rollback()
+        raise
+    finally:
+        psycopg_session.execute_script("DROP TABLE IF EXISTS test_table")
